;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy	
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "cpctelera.h.s"    ;;CPCtelera simbolos
.include "entity.h.s"
.include "hero.h.s"
.include "iaengine.h.s"

;;Direccion de sprites burbuja
.globl _bubble1_sp
.globl _bubble2_sp_0
.globl _bubble2_sp_1
.globl _bubble2_sp_2
.globl _bubble2_sp_3
.globl _bubble2_sp_4

;;===============================================================
;;===============================================================
;; PRIVATE DATA
;;===============================================================
;;===============================================================

;;Bubble DATA
.macro defineBubbleEntity name, x, y, w, h, vx, vy
    name'_data:
        defineDrawEntity name'_dw, x, y, w, h     ;;Parte dibujable
        name'_vx:        .db vx                   ;;Vel x
        name'_vy:        .db vy                   ;;Vel y
        name'_active:    .db #0                   ;;Activa (0:Desactivada,1:Activada)
        name'_maxdist_x: .db #0                   ;;Maxima distancia a la que llega en x
        name'_dire:      .db #0                   ;;Direccion (0: se mueve en x, 1: se mueve en y)
        name'_maxdist_y: .db #0                   ;;Maxima distancia a la que llega en y
        name'_state:     .db #0                   ;;Estado de animacion (0: burbuja_pequenya, 1: burbuja_grande
                                                  ;; 5: burbuja_explota)
        name'_anteriorx: .db  x                   ;; Guarda la x antigua
        name'_anteriory: .db  y                   ;; Guarda la y antigua
        name'_colision:  .db #0                   ;; 0: no ha colisionado, 1: ha colisionado
        name'_typeenem:  .db #0                   ;; tipo de enemigo (2: rata, 3:gallina, 4:rana)
.endm

;;Constantes de las estructuras de datos
.equ ent_vx, 4
.equ ent_vy, 5
.equ ent_act, 6
.equ ent_dis, 7
.equ ent_dir, 8
.equ ent_disy, 9
.equ ent_sta, 10
.equ ent_antx, 11
.equ ent_anty, 12
.equ ent_col, 13
.equ ent_ene, 14

;;Constantes de los datos de las burbujas
num_bub = 5                       ;; Numero de burbujas [Esto si se cambia cambiarlo en iaengine HayColision]
tam_bub = 15                      ;; Numero de datos de cada entidad [Esto si se cambia cambiarlo en iaengine HayColision]
burbuja_actual:  .dw #bub1_data   ;; Burbuja actual
contador_burbu:  .db #num_bub     ;; contador de burbujas

;;Datos de burbujas
defineBubbleEntity bub1  0 0 #0x04 #0x08 1 1
defineBubbleEntity bub2  0 0 #0x04 #0x08 1 1
defineBubbleEntity bub3  0 0 #0x04 #0x08 1 1
defineBubbleEntity bub4  0 0 #0x04 #0x08 1 1
defineBubbleEntity bub5  0 0 #0x04 #0x08 1 1

;;Contador burbuja_explota
tiempo_bub_explota: .db #5

;;===============================================================
;;===============================================================
;; PUBLIC FUNCTION
;;===============================================================
;;===============================================================
;;===============================================================
;; Inicialize bubbles
;; DESTROY: -
;; RETURNS: -
;;===============================================================
bubble_init::
    ld   a, #num_bub         ;; Numero de burbujas a dibujar
    ld  iy, #bub1_data       ;; IY Puntero a los datos de la primera burbuja
    call setPosicionBurbujas ;; Le damos el puntero a la primera burbuja a iaengine para que nos compruebe colisiones con los enemigos

    bucle_init_bub:
        push af              ;; Guardar el contador A

        ld  ent_x(iy),  #0
        ld  ent_y(iy),  #0
        ld  ent_w(iy),  #4
        ld  ent_h(iy),  #8
        ld  ent_vx(iy), #1
        ld  ent_vy(iy), #1
        ld  ent_act(iy), #0
        ld  ent_dis(iy), #0
        ld  ent_dir(iy), #0
        ld  ent_disy(iy), #0
        ld  ent_sta(iy), #0
        ld  ent_antx(iy), #0
        ld  ent_anty(iy), #0

        ld   bc, #tam_bub    ;; tam_bub
        add  iy, bc          ;; bub_data + tam_bub = siguiente burbuja
        pop  af              ;; Cargar el contador en A
        dec  a
        jr  nz, bucle_init_bub   ;; Dibujar tantas burbujas como num_bub
    ret

;;===============================================================
;; Llama a dibujar burbujas
;; IX -> Pointer to entity DATA
;; DESTROY: AF, BC, DE, HL
;;===============================================================
bubble_draw::
    ld   a, #num_bub         ;; Numero de burbujas a dibujar
    ld  iy, #bub1_data       ;; IX Puntero a los datos de la primera burbuja

    bucle_draw_bub:
        push af              ;; Guardar el contador A

        call drawBubble

        ld   bc, #tam_bub    ;; tam_bub
        add  iy, bc          ;; bub_data + tam_bub = siguiente burbuja
        pop  af              ;; Cargar el contador en A
        dec  a
        jr  nz, bucle_draw_bub   ;; Dibujar tantas burbujas como num_bub
    ret

;;===============================================================
;; Llama a borrar burbujas
;; IX -> Pointer to entity DATA
;; DESTROY: AF, BC, DE, HL
;;===============================================================
bubble_erase::
    ld   a, #num_bub         ;; Numero de burbujas a dibujar
    ld  iy, #bub1_data       ;; IX Puntero a los datos de la primera burbuja

    bucle_erase_bub:
        push af              ;; Guardar el contador A

        call eraseBubble

        ld   bc, #tam_bub    ;; tam_bub
        add  iy, bc          ;; bub_data + tam_bub = siguiente burbuja
        pop  af              ;; Cargar el contador en A
        dec  a
        jr  nz, bucle_erase_bub  ;; Borra tantas burbujas como num_bub
    ret

;;===============================================================
;; Llama a update burbujas
;; IX -> Pointer to entity DATA
;; DESTROY: AF, BC, DE, HL
;;===============================================================
bubble_update::

    ld   a, #num_bub         ;; Numero de burbujas a dibujar
    ld  iy, #bub1_data       ;; IX Puntero a los datos de la primera burbuja

    bucle_update_bub:
        push af              ;; Guardar el contador A

        call updateBubble

        ld   bc, #tam_bub       ;; tam_bub
        add  iy, bc             ;; bub_data + tam_bub = siguiente burbuja
        pop  af                 ;; Cargar el contador en A
        dec  a
        jr  nz, bucle_update_bub    ;; Actualizar tantas burbujas como num_bub
    ret

;;===============================================================
;; Disparo de burbuja del heroe
;; DESTROY: -
;;===============================================================
heroBubble::
    ;;Si estas en el aire no pudes disparar
    ld  a, ent_dis(ix)
    cp  #3       ;;Bajando izq
    jr  z, no_cambia_estado
    cp  #10       ;;Bajando der
    jr  z, no_cambia_estado
    cp  #2       ;;Saltando izq
    jr  z, no_cambia_estado
    cp  #9       ;;Saltando der
    jr  z, no_cambia_estado

        ;;Estado de disparar del heroe
        ld   b, #4
        ld   c, #11
        call  changeState
        no_cambia_estado:

        ld  iy, (burbuja_actual)       ;; IX Puntero a los datos de la primera burbuja

        ld   ent_dir(iy), #0      ;; Iniciar burbuja en 0 (se mueve en x sprite1)

        ;;Cambiar la direccion de la burbuja
        ld  a, ent_dir(ix)        ;; direccion donde mira la entidad
        cp  #0                    ;; A == 0, la entidad mira a la derecha
        jr  z, disp_derecha       ;; si A==0, dispara_derecha
            ;; Disparo a la izquierda
            ld  ent_vx(iy), #-2   ;; La burbuja en X
            ld   c, #-20           ;; Distancia maxima izquierda
            jr  nz, disp_izquierda
        disp_derecha:
            ;; Disparo a la derecha
            ld  ent_vx(iy), #2    ;; La burbuja en X
            ld   c, #20           ;; Distancia maxima derecha
        disp_izquierda:
        ld  ent_vy(iy), #1        ;; La burbuja en y

        ;;Conseguir distancia maxima donde llega en x
        ld   a, ent_x(ix)     ;; X del heroe
        add  a, c             ;; Conseguir distancia maxima A=ent_x + c
        ld  ent_dis(iy), a    ;; Maxima distancia a la que llega la burbuja

        ;;Conseguir distancia maxima donde llega en y
        ld   c, #-50          ;; Distancia maxima arriba
        ld   a, ent_y(ix)     ;; Y del heroe
        add  a, c             ;; Conseguir distancia maxima A=ent_y + c
        ld  ent_disy(iy), a   ;; Maxima distancia a la que llega la burbuja

        ;;Cambiar posicion de la burbuja donde el heroe
            ld  a, ent_x(ix)      ;;Pos x heroe
            ld  b, ent_y(ix)      ;;Pos y heroe

            ld  c, #2             ;;|
            add a, c              ;;\ heroe_x + c
            push af               ;; guardar heroe_x

            ld  c, #3             ;;|
            ld  a, ent_y(ix)      ;;|
            add a, c              ;;\ heroe_y + c

            ld  ent_y(iy), a      ;; y burbuja = y heroe
            pop af                ;; A = heroe_y
            ld  ent_x(iy), a      ;; x burbuja = x heroe

        ld  ent_act(iy), #1       ;; Activar la burbuja

        ;;Comprobar que es la ultima burbuja
        ld   a, (contador_burbu)   ;; A = contador_burbu
        dec  a                     ;; contador_burbu--
        ld   (contador_burbu), a   ;; contador_burbu = A
        cp  #0                     ;; If (contador_burbu == 0)
        jr   z, ultima_burbuja     ;; Saltar si es la ultima burbuja

            ;;Cambiar a burbuja siguiente
            ld   bc, #tam_bub           ;; tam_bub
            add  iy, bc                 ;; bub_data + tam_bub = siguiente burbuja
            ld  (burbuja_actual), iy    ;; Siguiente burbuja

    ret

    ultima_burbuja:
    ;;Reiniciamos contador
    ld   a, #num_bub            ;; |
    ld   (contador_burbu), a    ;; \ contador_burbu = num_bub

    ;;Actual es la primera burbuja
    ld  hl, #bub1_data
    ld  (burbuja_actual), hl

    ret

;;La burbuja captura al enemigo
BurbujaEnemigo::
    ld  ent_col(iy), #1         ;; 1: ha colisionado
    inc a
    inc a                       ;; Para ajustar a los sprites (2: rata, 3:gallina, 4:rana)
    ld  ent_ene(iy), a          ;; Tipo de enemigo
    ld  ent_x(iy), b            ;; Ajustar x burbuja

    ret
;;===============================================================
;;===============================================================
;; PRIVATE FUNCTION
;;===============================================================
;;===============================================================

;;===============================================================
;; Dibujar burbuja
;; IX -> Pointer to entity DATA
;; DESTROY: AF, BC, DE, HL
;;===============================================================
drawBubble:
    ld  a, ent_act(iy)               ;;If(ent_act(ix) == 0 desactivada)
    cp  #0
    jr  z, no_activada

        ;;Dibujar burbuja
        call selectSprite
        ld  a, #1                ;; si es 0 (dibujar en ix), si es 1 (dibujar en iy)
        call drawEntity

    no_activada:

    ret

;;===============================================================
;; Borrar burbuja
;; IX -> Pointer to entity DATA
;;           H: old x to erase
;;           L: old y to erase
;; DESTROY: AF, BC, DE, HL
;;===============================================================
eraseBubble:
    ld  a, ent_act(iy)               ;;If(ent_act(ix) == 0 desactivada)
    cp  #0
    jr  z, no_activada3
        ld  a, #1                ;; si es 0 (borrar en ix), si es 1 (borrar en iy)
        ld  h, ent_antx(iy)
        ld  l, ent_anty(iy)
        call eraseEntity
        ret
    no_activada3:
    ld  a, #1                ;; si es 0 (borrar en ix), si es 1 (borrar en iy)
    ld  h, ent_antx(iy)
    ld  l, ent_anty(iy)
    call eraseEntity
    ld  ent_col(iy), #0
    ld  ent_ene(iy), #0
    ld  ent_x(iy), #0       ;;inicializar posicion
    ld  ent_y(iy), #0       ;;inicializar posicion
    ld  ent_w(iy), #4       ;;inicializar tamanyo
    ld  ent_h(iy), #8       ;;inicializar tamanyo
    ret

;;===============================================================
;; Update la burbuja, su movimiento y colisiones
;; DESTROY: -
;;===============================================================
updateBubble:
    ld  a, ent_act(iy)      ;; If(ent_act(ix) == 0 desactivada)
    cp  #0
    jr  z, no_activada2

        ;;Guardar posicion de la burbuja para borrado en buffer
        ld  a, ent_x(iy)
        ld  ent_antx(iy), a
        ld  a, ent_y(iy)
        ld  ent_anty(iy), a

        ;;Disparo en x
        ld  ent_sta(iy), #0      ;; entity state = 0 (burbuja_pequenya)

        ;;Colision con enemigo, moverse en y
        ld   a, ent_col(iy)
        cp  #1
        jr  z, moverse_en_y

        ;;Comprobar limites del mapa
        ld   a, ent_x(iy)               ;; A = entity_x
        cp   #80 - 11                   ;; limite derecho
        jr   z, moverse_en_y            ;; no mover en x, mover en y
        cp   #80 - 10                   ;; limite derecho si es un bit mas
        jp   z, restar_bit_y_moverse_y  ;; colocar un bit menos y no mover en x, mover en y
        cp   #4                         ;; limite izquierdo
        jp   z, moverse_en_y            ;; no mover en x, mover en y
        cp   #3                         ;; limite izquierdo si es un bit menos
        jp   z, sumar_bit_y_moverse_y   ;; colocar un bit mas y no mover en x, mover en y

            add  ent_vx(iy)             ;;|
            ld   ent_x(iy), a           ;;\ entity_x + (2 x entity_vx)

            cp   ent_dis(iy)            ;; distancia maxima
            jr   nz, no_activada2

                ;;Disparo en y
                moverse_en_y:
                ld   ent_vx(iy), #0     ;; velocidad en x a 0
                ld   a, ent_y(iy)       ;; a = entity_y

                ;;Cambiar tamanyo de la burbuja
                ld  ent_w(iy), #0x08    ;;burbuja grande
                ld  ent_h(iy), #0x10    ;;burbuja grande

                ;;Burbuja si no ha colisionado
                ld   a, ent_col(iy)
                cp  #0
                jr  nz, sprite_conenemigo
                    ld  ent_sta(iy), #1     ;; entity state = 0 (burbuja_grande)
                    jr continuar_sprite
                sprite_conenemigo:
                    ld  b, ent_ene(iy)
                    ld  ent_sta(iy), b         ;; entity state = 2 (burbuja_pollo)

                continuar_sprite:
                ;;Comprobar techo del mapa
                ld   a, ent_y(iy)       ;; a = entity_y
                cp   #16                      ;; limite techo
                jr   z, desactivar_burbu     ;; no mover en y y descativar burbuja

                    ;;Comprobar limite de distancia en y
                    cp  ent_disy(iy)         ;; limite en y de la burbuja
                    jr  z, desactivar_burbu  ;; desactivar burbuja

                        sub  ent_vy(iy)         ;;|
                        ld   ent_y(iy), a       ;;\ entity_y - entity_vy

                        ;;Cambiar direccion para sprite
                        ld   ent_dir(iy), #1

    no_activada2:
    ret

    desactivar_burbu:
    ;;Tiempo en explotar
    ld  a, (tiempo_bub_explota)
    dec a
    ld (tiempo_bub_explota), a
    jr z, desactivar_burbuja
        ld  ent_sta(iy), #5     ;; entity state = 5 (burbuja_explota)
        ret
    desactivar_burbuja:
    ld  ent_act(iy), #0         ;;Desactivar la burbuja
    ld  a, #5
    ld  (tiempo_bub_explota), a ;;Reiniciar contador

    ;;contar puntos
    call contarPuntos
    ret

    restar_bit_y_moverse_y:
    ld   a, ent_x(iy)       ;; A = entity_x
    dec  a                  ;; A--
    ld   ent_x(iy), a       ;; entity_x = A
    jp   moverse_en_y       ;; saltar a moverse_en_y

    sumar_bit_y_moverse_y:
    ld   a, ent_x(iy)       ;; A = entity_x
    inc  a                  ;; A++
    ld   ent_x(iy), a       ;; entity_x = A
    jp   moverse_en_y       ;; saltar a moverse_en_y


contarPuntos:
    ld  a, ent_ene(iy)
    cp  #2                  ;;2: rata
    jr  nz, no_es_rata
        ld  b, #150         ;;150
        jr terminar_puntos
    no_es_rata:
    cp  #3                  ;;3:gallina
    jr  nz, no_es_gallina
        ld   b, #200        ;;200
        jr terminar_puntos
    no_es_gallina:
    cp  #4                  ;;4:rana
    jr  nz, no_hay_enemigo
        ld   b, #50        ;;100
        jr terminar_puntos
    no_hay_enemigo:
        ld   b, #5         ;;5
    terminar_puntos:
    call sumarPuntos

    ret
;;===============================================================
;; Devolver sprite dependiendo del estado de las burbujas
;; Return:
;;          HL -> Pointer to Sprite
;; DESTROY: AF, HL
;;===============================================================
selectSprite:
    ;;Comprobar estado de animaciones
    ld  a, ent_sta(iy)          ;; A = entity_state
    cp  #0                      ;; A == 0? (0 burbuja_pequenya)
    jr  z, burbuja_pequenya
    cp  #1                      ;; A == 1? (1 burbuja_grande)
    jr  z, burbuja_grande
    cp  #2                      ;; A == 2? (2 burbuja_pollo)
    jr  z, burbuja_raton
    cp  #3                      ;; A == 3? (3 burbuja_raton)
    jr  z, burbuja_pollo
    cp  #4                      ;; A == 4? (4 burbuja_rana)
    jr  z, burbuja_rana
    cp  #5                      ;; A == 5? (5 esta saltando der)
    jr  z, burbuja_explota

    ;;Dibujar sprite dependiendo del hero_state
    burbuja_pequenya:
    ld  hl, #_bubble1_sp        ;;HL = Sprite
    ret

    burbuja_grande:
    ld  hl, #_bubble2_sp_0        ;;HL = Sprite
    ret

    burbuja_raton:
    ld  hl, #_bubble2_sp_2        ;;HL = Sprite
    ret

    burbuja_pollo:
    ld  hl, #_bubble2_sp_1        ;;HL = Sprite
    ret

    burbuja_rana:
    ld  hl, #_bubble2_sp_3        ;;HL = Sprite
    ret

    burbuja_explota:
    ld  hl, #_bubble2_sp_4        ;;HL = Sprite
    ret
