;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy	
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

;;===============================================================
;; Check collision between 2 entitys
;; Input:
;;      HL: Point to the first entity (entityA) entityA
;;      DE: Point to the second entity (entityB) entityB
;; Return:-
;;      A: 0: no_collision, 1: si_collision
;;===============================================================

checkCollision::
    ;; entityB to the left of entityA
    ;; If (entityB_x + entityB_w <= entityA_x) no_colission
    ;;
    ;; entityB_x + entityB_w - entityA_x <= 0
    ld   a, (de)                ;; A = entityB_x
    ld   c, a                   ;; C = entityB_x
    inc  de                     ;; | entityB_y
    inc  de                     ;; | entityB_w
    ld   a, (de)                ;; A = entityB_w
    add  c                      ;; A = entityB_x + entityB_w
    sub  (hl)                   ;; entityA_x
    jr   z, no_colission        ;; If( <= 0)
    jp   m, no_colission        ;; then no_colission

    ;; entityB to the right of entityA
    ;; If (entityA_x + entityA_w <= entityB_x)
    ;;
    ;; entityA_x + entityA_w - entityB_x <= 0
    ld   a, (hl)                ;; A = entityA_x
    inc  hl                     ;; | entityA_y
    inc  hl                     ;; | entityA_w
    add  (hl)                   ;; A = entityA_x + entityA_w
    ld   c, a                   ;; C = entityA_x + entityA_w
    dec  de                     ;; | entityB_y
    dec  de                     ;; | entityB_x
    ld   a, (de)                ;; A = entityB_x
    ld   b, a                   ;; B = entityB_x
    ld   a, c                   ;; A = entityA_x + entityA_w
    sub  b                      ;; (entityA_x + entityA_w) - entityB_x
    jr   z, no_colission        ;; If( <= 0)
    jp   m, no_colission        ;; then no_colission

    ;; entityB below entityA
    ;; If (entityB_y + entityB_h <= entityA_y) no_colission
    ;;
    ;; entityB_y + entityB_h - entityA_y <= 0
    inc  de                     ;; | entityB_y
    ld   a, (de)                ;; A = entityB_y
    ld   c, a                   ;; C = obstcale_y
    inc  de                     ;; | entityB_w
    inc  de                     ;; | entityB_h
    ld   a, (de)                ;; A = entityB_h
    add  c                      ;; A = entityB_h + entityB_y
    dec  hl                     ;; | entityA_y
    sub  (hl)                   ;; entityB_y + entityB_h - entityA_y
    jr   z, no_colission        ;; If( <= 0)
    jp   m, no_colission        ;; then no_colission

    ;; entityB above entityA
    ;; If (entityA_y + entityA_h <= entityB_y) no_collision
    ;;
    ;; entityA_y + entityA_h - entityB_y <= 0
    ld   a, (hl)                ;; A = entityA_y
    inc  hl                     ;; | entityA_w
    inc  hl                     ;; | entityA_h
    add  (hl)                   ;; entityA_y + entityA_h
    ld   c, a                   ;; C = entityA_y + entityA_h
    dec  de                     ;; | entityB_w
    dec  de                     ;; | entityB_y
    ld   a, (de)                ;; A = entityB_y
    ld   b, a                   ;; B = entityB_y
    ld   a, c                   ;; A = entityA_y + entityA_h
    sub  b                      ;; (entityA_y + entityA_h) - entityB_y
    jr   z, no_colission        ;; If( <= 0)
    jp   m, no_colission        ;; then no_colission

    ;; Hay colision
    ld  a, #1
    ret

    ;; No hay colision
    no_colission:
    ld  a, #0
    ret

checkCollision_Plataform::
    ;; entityB to the left of entityA
    ;; If (entityB_x + entityB_w <= entityA_x) no_colission
    ;;
    ;; entityB_x + entityB_w - entityA_x <= 0
    ld   a, (de)                ;; A = entityB_x
    ld   c, a                   ;; C = entityB_x
    inc  de                     ;; | entityB_y
    inc  de                     ;; | entityB_w
    ld   a, (de)                ;; A = entityB_w
    add  c                      ;; A = entityB_x + entityB_w
    sub  (hl)                   ;; entityA_x
    ;jr   z, no_colission        ;; If( <= 0)
    jp   m, no_colission        ;; then no_colission

    ;; entityB to the right of entityA
    ;; If (entityA_x + entityA_w <= entityB_x)
    ;;
    ;; entityA_x + entityA_w - entityB_x <= 0
    ld   a, (hl)                ;; A = entityA_x
    inc  hl                     ;; | entityA_y
    inc  hl                     ;; | entityA_w
    add  (hl)                   ;; A = entityA_x + entityA_w
    ld   c, a                   ;; C = entityA_x + entityA_w
    dec  de                     ;; | entityB_y
    dec  de                     ;; | entityB_x
    ld   a, (de)                ;; A = entityB_x
    ld   b, a                   ;; B = entityB_x
    ld   a, c                   ;; A = entityA_x + entityA_w
    sub  b                      ;; (entityA_x + entityA_w) - entityB_x
    ;jr   z, no_colission        ;; If( <= 0)
    jp   m, no_colission        ;; then no_colission

    ;; entityB above entityA
    ;; If (entityA_y + entityA_h != entityB_y) no_collision
    ;;
    ;; entityA_y + entityA_h - entityB_y != 0
    dec  hl                     ;; | entityA_y
    ld   a, (hl)                ;; A = entityA_y
    inc  hl                     ;; | entityA_w
    inc  hl                     ;; | entityA_h
    add  (hl)                   ;; entityA_y + entityA_h
    ld   c, a                   ;; C = entityA_y + entityA_h
    inc  de                     ;; | entityB_y
    ld   a, (de)                ;; A = entityB_y
    ld   b, a                   ;; B = entityB_y
    ld   a, c                   ;; A = entityA_y + entityA_h
    sub  b                      ;; (entityA_y + entityA_h) - entityB_y
    jr   nz, no_colission        ;; If( != 0)

    ;; Hay colision
    ld  a, #1
    ret
