;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy	
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "entity.h.s"
.include "cpctelera.h.s"
.include "hero.h.s"
.include "enemy.h.s"
.include "bubbles.h.s"
.include "obstacles.h.s"
.include "map.h.s"

front_buffer: .db 0xC0
back_buffer:  .db 0x80

;;===============================================================
;; Limpia y cambia el buffer
;;===============================================================
newScene:
    call switchBuffers
    call clearBuffers
    ret

;;===============================================================
;; Inicializar el back buffer
;;===============================================================
InitBuffers:
    ld   hl, #0x8000       ;; Direccion donde meter los 0
    ld  (hl), #0           ;; 0 a meter
    ld   de, #0x8000 + 1   ;; Siguiente bite de direccion
    ld   bc, #0x4000 - 1   ;; Tamanyo de la memoria que vamos a rellenar de 0
    ldir                   ;; Bucle

    ld   hl, #0xC000       ;; Direccion donde meter los 0
    ld  (hl), #0           ;; 0 a meter
    ld   de, #0xC000 + 1   ;; Siguiente bite de direccion
    ld   bc, #0x4000 - 1   ;; Tamanyo de la memoria que vamos a rellenar de 0
    ldir                   ;; Bucle

    ret

;;===============================================================
;; Limpiar el back buffer
;;===============================================================
clearBuffers:

    call bubble_erase
    call hero_erase
    call enemy_eraseGroup
    call obstacle_erase

    ld  a, (back_buffer)        ;; A=back_buffer
    ld  h, a                    ;; D=A
    ld  l, #0                   ;; DE = 0xC000 o 0x8000
    call map_DrawObjects

    ret

;;===============================================================
;; Cambiar los buffer
;;===============================================================
switchBuffers:
    ;;Intercambiar front_buffer <-> back_buffer
    ld  a, (back_buffer)
    ld  b, a
    ld  a, (front_buffer)
    ld  (back_buffer), a
    ld  a, b
    ld  (front_buffer), a

    srl  b     ;; |
    srl  b     ;; \ Desplaza a la izquierda 2 bits
    ld  l, b
    jp cpct_setVideoMemoryPage_asm

;;===============================================================
;; Dibujar una entidad
;; Input:
;;        A: si es 0 (dibujar en ix), si es 1 (dibujar en iy)
;;    IX/IY: Pointer to entity DATA
;;       HL: Pointer to entity sprite
;; DESTROY: AF, BC, DE, HL
;;===============================================================
drawEntity:
    push hl

    ;;Comprobar si es en ix o iy
    cp #0
    jr nz, en_iy
        ;;EN IX
        ;;Calcular la posicion en pantalla
        ld  a, (back_buffer)        ;; A=back_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        ld  c, ent_x(ix)            ;; C=entity_x
        ld  b, ent_y(ix)            ;; B=entity_y
        call cpct_getScreenPtr_asm

        ;;Dibujar entidad
        ex de, hl                   ;; Intercambia hl por de (para dibujar el sprite en esa posicion)
        ld  c, ent_w(ix)            ;; C=entity_w
        ld  b, ent_h(ix)            ;; B=entity_h
        pop hl                      ;; HL=Pointer to Sprite
        call cpct_drawSpriteMasked_asm    ;; Draw Sprite with mask

    ret
    en_iy:
        ;;EN IY
        ;;Calcular la posicion en pantalla
        ld  a, (back_buffer)        ;; A=back_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        ld  c, ent_x(iy)            ;; C=entity_x
        ld  b, ent_y(iy)            ;; B=entity_y
        call cpct_getScreenPtr_asm

        ;;Dibujar entidad
        ex de, hl                   ;; Intercambia hl por de (para dibujar el sprite en esa posicion)
        ld  c, ent_w(iy)            ;; C=entity_w
        ld  b, ent_h(iy)            ;; B=entity_h
        pop hl                      ;; HL=Pointer to Sprite
        call cpct_drawSpriteMasked_asm    ;; Draw Sprite with mask
    ret
;;===============================================================
;; Borrar una entidad en iy
;; Input:
;;        A: si es 0 (borrar en ix), si es 1 (borrar en iy), si es 3 (borra en ix en front buffer),si es 4 (borra en iy en front buffer)
;;    IX/IY: Pointer to entity DATA
;;        H: old x to draw
;;        L: old y to draw
;; DESTROY: AF, BC, DE, HL
;;===============================================================
eraseEntity:
    push hl

    ;;Comprobar si es en ix o iy
    cp #0
    jr nz, en_iy2
        ;;EN IX
        ;;Calcular la posicion en pantalla
        ld  a, (back_buffer)        ;; A=back_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        pop hl
        ld  c, h                    ;; C=entity_antx
        ld  b, l                    ;; B=entity_anty
        call cpct_getScreenPtr_asm

        ;;Dibujar fondo
        ex  de, hl                 ;; Intercambia hl por de
        ld  a, #0x3c                  ;; Color de fondo
        ld  b, ent_h(ix)           ;; B=Alto
        ld  c, ent_w(ix)           ;; C=Ancho
        call cpct_drawSolidBox_asm  ;; (esta en bytes)

    ret
    en_iy2:
        ;; comprobamos que si es tres se pase a la funcion si no que siga con este sea quien sea
        cp #3
        jr z, en_ix3;; se va front buffer de x
        cp #4
        jr z, en_iy4;; se va front buffer de y
        ;;EN IY
        ;;Calcular la posicion en pantalla
        ld  a, (back_buffer)        ;; A=back_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        pop hl
        ld  c, h                    ;; C=entity_antx
        ld  b, l                    ;; B=entity_anty
        call cpct_getScreenPtr_asm

        ;;Dibujar fondo
        ex  de, hl                 ;; Intercambia hl por de
        ld  a, #0x3c                  ;; Color de fondo
        ld  b, ent_h(iy)           ;; B=Alto
        ld  c, ent_w(iy)           ;; C=Ancho
        call cpct_drawSolidBox_asm  ;; (esta en bytes)

    ret

    en_ix3:
        ;;EN IX
        ;;Calcular la posicion en pantalla
        ld  a, (front_buffer)       ;; A=front_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        pop hl
        ld  c, h                    ;; C=entity_antx
        ld  b, l                    ;; B=entity_anty
        call cpct_getScreenPtr_asm

        ;;Dibujar fondo
        ex  de, hl                 ;; Intercambia hl por de
        ld  a, #0x3c                  ;; Color de fondo
        ld  b, ent_h(ix)           ;; B=Alto
        ld  c, ent_w(ix)           ;; C=Ancho
        call cpct_drawSolidBox_asm  ;; (esta en bytes)
    ret

    en_iy4:
        ;;EN IY
        ;;Calcular la posicion en pantalla
        ld  a, (front_buffer)        ;; A=front_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        pop hl
        ld  c, h                    ;; C=entity_antx
        ld  b, l                    ;; B=entity_anty
        call cpct_getScreenPtr_asm

        ;;Dibujar fondo
        ex  de, hl                 ;; Intercambia hl por de
        ld  a, #0x3c                  ;; Color de fondo
        ld  b, ent_h(iy)           ;; B=Alto
        ld  c, ent_w(iy)           ;; C=Ancho
        call cpct_drawSolidBox_asm  ;; (esta en bytes)
    ret

;;===============================================================
;; Dibujar un string
;; Input:
;;          H: X position
;;          L: Y position
;;          D: Colour text
;;          E: Colour background
;;         IY: string
;;          A: select a buffer (0: buffer trasero, 1: buffer delantero)
;; DESTROY: AF, BC, DE, HL
;;===============================================================

drawString:
    push hl
    push af

    ;;Set colour and background
    ld  l, d
    ld  h, e
    call cpct_setDrawCharM0_asm

    pop af
    cp #0
    jr  z, elegido_Btrasero
        ;;delantero
        ;;Set pointer to screen
        ld  a, (front_buffer)        ;; A=front_buffer
        ld  d, a                    ;; D=A
        ld  e, #0                   ;; DE = 0xC000 o 0x8000
        jr continuar_delantero
    elegido_Btrasero:
    ;;Set pointer to screen
    ld  a, (back_buffer)        ;; A=back_buffer
    ld  d, a                    ;; D=A
    ld  e, #0                   ;; DE = 0xC000 o 0x8000
    continuar_delantero:
    pop hl
    ld  c, h                    ;; X
    ld  b, l                    ;; Y
    call cpct_getScreenPtr_asm

    ;;Draw String
    call cpct_drawStringM0_asm

    ret
