;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "keyboard/keyboard.h.s"
.include "cpctelera.h.s"    ;;CPCtelera simbolos
.include "entity.h.s"
.include "plataform.h.s"
.include "iaengine.h.s"
.include "bubbles.h.s"
.include "main.h.s"
.include "interfaz.h.s"

;;Direccion de sprites hero
.globl _snake_sp_13
.globl _snake_sp_12
.globl _snake_sp_11
.globl _snake_sp_10
.globl _snake_sp_09
.globl _snake_sp_08
.globl _snake_sp_07
.globl _snake_sp_06
.globl _snake_sp_05
.globl _snake_sp_04
.globl _snake_sp_03
.globl _snake_sp_02
.globl _snake_sp_01
.globl _snake_sp_00
;;===============================================================
;;===============================================================
;; PRIVATE DATA
;;===============================================================
;;===============================================================

;;Hero DATA
.macro defineHeroEntity name, x, y, w, h, vx, vy, jmp, st, dr, gr, startC, endC
    name'_data:
        defineDrawEntity name'_dw, x, y, w, h  ;;Parte dibujable
        name'_vx:        .db vx           ;; Vel x
        name'_vy:        .db vy           ;; Vel y
        name'_jump:      .db jmp          ;; Indice de la tabla (-1 inactivo, -2 bajando)
        name'_state:     .db st           ;; Estado de animacion (0: suelo_izq, 1: salta_izq, 2: baja_izq, 3: mueve_izq
                                          ;;                     4: suelo_der, 5: salta_der, 6: baja_der, 7: mueve_der
                                          ;;                     8: disp_izq,  9: disp_der
                                          ;;                    10: atontado1, 11: atontado2 )
        name'_dire:      .db dr           ;; Direccion que mira el heroe: (0:mira_derecha, 1:mira_izquierda)
        name'_grav:      .db gr           ;; Gravedad (0:gravedad_active, 1:gravedad_inactive)
        name'_startColl: .dw startC       ;; donde esta la funcion para que la llame coll
        name'_endColl:   .dw  endC        ;; donde esta la funcion para que la llame coll
        name'_anteriorx: .db  x           ;; Guarda la x antigua
        name'_anteriory: .db  y           ;; Guarda la y antigua
        name'_vida:      .db  #3          ;; 3 vidas al comienzo, 0 vidas estas muerto
        name'_anim:      .db  #6          ;; tiempo de animacion
        name'_point:     .dw  #0          ;; puntos de la partida
        name'_contizq:   .db  #5          ;; Contador de animacion izquierda
        name'_contder:   .db  #5          ;; Contador de animacion derecha

.endm

;;Constantes de las estructuras de datos
.equ ent_vx, 4
.equ ent_vy, 5
.equ ent_jmp, 6
.equ ent_sta, 7
.equ ent_dir, 8
.equ ent_gra, 9
.equ ent_scL, 10 ;byte bajo
.equ ent_scH, 11 ;byte alto
.equ ent_ecL, 12 ;byte bajo
.equ ent_scH, 13 ;byte alto
.equ ent_antx, 14
.equ ent_anty, 15
.equ ent_vida, 16
.equ ent_anim, 17
.equ ent_pnt_l,  18
.equ ent_pnt_h,  19
.equ ent_cd, 20
.equ ent_ci, 21

;;Datos de Heroe
defineHeroEntity hero 40 176 #0x08 #0x10 1 1 -1 0 0 0 hero_startCollision hero_endCollision

;;Tabla de saltos
jumptable:
    .db  #-8, #-7, #-6, #-6
    .db  #-5, #-5, #-4, #-4
    .db  #-3, #-3, #-2, #-2
    .db  #-1, #-1, #00, #00
    .db  #0x80

;; Contadores
contadorespacio: .db #15

;; Flags
desactivar_teclado: .db #1     ;; (0: teclado desactivado, 1: teclado activado)
game_over_control:: .db #0     ;; (0: NO es game over, 1: ES game over)

;;===============================================================
;;===============================================================
;; PUBLIC FUNCTION
;;===============================================================
;;===============================================================

;;===============================================================
;; Inicialize hero
;; DESTROY: -
;; RETURNS: -
;;===============================================================
first_init_hero::
    ld ix, #hero_data      ;; IX Puntero a los datos
    ld ent_vida(ix), #3
    ld ent_pnt_l(ix), #0
    ld ent_pnt_h(ix), #0
    ret
hero_init::
    push af
    ld ix, #hero_data      ;; IX Puntero a los datos
    call setPosicionJugador ;; le da a la ia su puntero

    ld ix, #hero_data      ;; IX Puntero a los datos

    ;;Si estabas muerto iniciar puntos y vidas
    ld   a, (game_over_control)
    cp  #0
    jr  z, noinit_puntosvida
        ld ent_vida(ix), #3

        ;;Perder puntos
        ;;Bajas 1 a H si no puedes bajar porque es 0, bajar a 0 L
        ld  a, ent_pnt_h(ix)
        cp  #0
        jr  z, no_puedo_bajarmas
            dec  a
            ld ent_pnt_h(ix), a  ;;Bajas 256 puntos
            ld   a, ent_pnt_l(ix)
            ld   b, #6
            add  a, b            ;;Bajas exactamente 250 puntos
            ld   ent_pnt_l(ix), a
            jr puntos_bajados
        no_puedo_bajarmas:
            ld ent_pnt_l(ix), #0
            ld ent_pnt_h(ix), #0
        puntos_bajados:

    noinit_puntosvida:

    ;;Sumar uno de vida si no tienes 3
    ld  a, ent_vida(ix)
    cp #3
    jr z, no_sum_vida
        inc a
        ld ent_vida(ix), a
    no_sum_vida:

    pop af
    ;;level 0
    cp #0
    jr nz, hero_level1

        ld ent_x(ix), #40
        ld ent_y(ix), #176
        ld ent_dir(ix), #0

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level1:
    ;;level 1
    cp #1
    jr nz, hero_level2

        ld ent_x(ix), #60
        ld ent_y(ix), #134
        ld ent_dir(ix), #1

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level2:
    ;;level 2
    cp #2
    jr nz, hero_level3

        ld ent_x(ix), #60
        ld ent_y(ix), #176
        ld ent_dir(ix), #1

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level3:
    ;;level 3
    cp #3
    jr nz, hero_level4

        ld ent_x(ix), #65
        ld ent_y(ix), #176
        ld ent_dir(ix), #0

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level4:
    ;;level 4
    cp #4
    jr nz, hero_level5

        ld ent_x(ix), #62
        ld ent_y(ix), #176
        ld ent_dir(ix), #0

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level5:
    ;;level 5
    cp #5
    jr nz, hero_level6

        ld ent_x(ix), #10
        ld ent_y(ix), #176
        ld ent_dir(ix), #0

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level6:
    ;;level 6
    cp #6
    jr nz, hero_level7

        ld ent_x(ix), #64
        ld ent_y(ix), #155
        ld ent_dir(ix), #1

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level7:
    ;;level 7
    cp #7
    jr nz, hero_level8

        ld ent_x(ix), #35
        ld ent_y(ix), #142
        ld ent_dir(ix), #1

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level8:
    ;;level 8
    cp #8
    jr nz, hero_level9

        ld ent_x(ix), #35
        ld ent_y(ix), #160
        ld ent_dir(ix), #1

        ;;Activar teclado
        ld  a, #1
        ld  (desactivar_teclado), a
        jp  terminar_init

    hero_level9:
    terminar_init:
    ;;inicializar cosas genericas
    ld  a, ent_x(ix)
    ld ent_antx(ix), a
    ld  a, ent_y(ix)
    ld ent_anty(ix), a
    ld ent_jmp(ix), #-1
    ld ent_sta(ix), #0
    ld ent_gra(ix), #0

    ;;iniciar game OVER
    ld  a, #0
    ld  (game_over_control), a

    ret

;;===============================================================
;; Start and End collision with plataform
;; DESTROY: A
;; RETURNS: -
;;===============================================================
hero_startCollision::
    ;;Si estas saltando no gravedad
    ld  a, ent_jmp(ix)      ;; A = entity_jump
    cp  #-2                 ;; A == -2? Is jump active?
    jr nz, not_end_grav     ;; Jump active, no gravedad

        ld ent_gra(ix), #1          ;; Desactivar gravedad
        jp z, end_of_jump           ;; Acabar salto

    not_end_grav:
    ret

hero_endCollision::
    ld  a, ent_gra(ix)            ;;|
    cp  #1                        ;;|
    jr  nz, no_cambiar_gravedad   ;; Si la gravedad esta desactivada..
        ld ent_gra(ix), #0        ;; Activar gravedad
    no_cambiar_gravedad:
    ret

;;===============================================================
;; Sumar puntos por burbuja
;; DESTROY: A
;; INPUT:
;;      B: Puntos a sumar
;;===============================================================
sumarPuntos::
    ld ix, #hero_data
    ld  a, b
    ld  c, a
    ld  b, #0
    ld  l, ent_pnt_l(ix)
    ld  h, ent_pnt_h(ix)
    add  hl, bc
    ld  ent_pnt_l(ix), l
    ld  ent_pnt_h(ix), h
    ret
;;===============================================================
;; Return get pointer
;; DESTROY: HL
;; RETURNS:
;;      HL: Pointer to hero data
;;===============================================================
hero_getPtrHL::
    ld   hl, #hero_dw_x    ;;Puntero hero_x
    ret
hero_getPtrIX::
    ld   ix, #hero_data    ;;Puntero al heroe
    ret
;;===============================================================
;; Erases the Hero
;;           H: old x to erase
;;           L: old y to erase
;; DESTROY: AF, BC, HL
;;===============================================================
hero_erase::
    ld ix, #hero_data      ;; IX Puntero a los datos
    ld  a, #0              ;; si es 0 (borrar en ix), si es 1 (borrar en iy)
    ld  h, ent_antx(ix)
    ld  l, ent_anty(ix)
    call eraseEntity       ;; Borrar nuestro heroe

    ret

;;===============================================================
;; Updates the Hero
;; DESTROY: AF, BC, HL
;;===============================================================
hero_update::
    ld ix, #hero_data      ;; IX Puntero a los datos

    ;;Guardar posicion del heroe para borrado en buffer
    ld  a, ent_x(ix)
    ld  ent_antx(ix), a
    ld  a, ent_y(ix)
    ld  ent_anty(ix), a

    call GameoverAnimation
    call setComprobando    ;; este paso se que se puede omitir pero de momento lo dejo
    call plataform_update

    ld  a, (controll_selected)
    cp #2
    jr z, es_joystick
        call checkUserInput    ;; Comprobar tecla
        jr es_teclado
    es_joystick:
        call checkUserInputjoy ;; comprobar botones joy
    es_teclado:

    call JumpControl       ;; Comprobar salto
    call gravityHero
    ret

;;===============================================================
;; Draws the Hero
;; DESTROY: AF, BC, HL
;; RETURNS:
;;          HL: Pointer to Sprite
;;           A: Draw in ix or iy
;;===============================================================
hero_draw::
    ld ix, #hero_data       ;; IX Puntero a los datos

    ;;Dibujado de heroe
    call selectSprite
    ld  a, #0               ;; si es 0 (dibujar en ix), si es 1 (dibujar en iy)
    call drawEntity

    ret

;;===============================================================
;; Change state of entity depending on where he look
;; INPUT:
;         B: Estado izquierda, C: Estado derecha
;; DESTROY: A,F,B,C
;;===============================================================
changeState::
    ld  a, ent_dir(ix)        ;; direccion donde mira la entidad
    cp  #0                    ;; A == 0, la entidad mira a la derecha
    jr  z, mira_derecha       ;; si A==0, mira_derecha

        ld  ent_sta(ix), b    ;; entity state = B (mirando izquierda)
        jr  mira_izq          ;; Solo a la izquierda

    mira_derecha:
        ld  ent_sta(ix), c    ;; entity state = C (mirando derecha)

    mira_izq:

    ret

;;===============================================================
;;Quita vida al jugador
;;Cambia el valor de vida del jugador ent_vida
;;LLamada desde atacar de Enemigo
;;===============================================================
Quitar_Vida::
    ld ix, #hero_data   ;; IX Puntero a los datos
    ld a, ent_vida(ix)  ;; Cargamos la vida en A
    cp #0               ;; Comparamos con 0 vidas
    jr z, game_over     ;; si tiene cero se llama a game over, si no se resta uno a las vidas
        dec a
        ld ent_vida(ix), a

        ;;Estado de quitar vida
        ld   b, #5
        ld   c, #12
        call  changeState
    ret
    game_over:                      ;; es llamado cuando tienes cero vidas
        call desactivar_enemigos    ;; desactivamos todos los enemigos

        ld  a, #0
        ld  (desactivar_teclado), a

        ld  a, #1
        ld  (game_over_control), a

        ;;Inicializar game over
        ld  a, #1
        ld  (game_over_time), a
ret

;; se hace aqui por ahorrar codigo, sirve para el menu detectar si se pulsa espacio empezar nivel.
;; si esta pulsado devuelve 1 si no 0 en el registro A
ComprobarEspacio::

ld  a, (controll_selected)
cp #2
jr  z, es_un_joistick

    call cpct_scanKeyboard_asm ;;escaneamos las teclas
    ld   hl, #Key_Space        ;; HL = Key_Space Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  Space == 0
    jr   z, space_no_pressed    ;; Saltar si Space == 1 (Space no esta pulsada)
        ld a, #1 ; si presionado
        ret
    space_no_pressed:
        ld a, #0 ; no presionado
        ret

es_un_joistick:

    call cpct_scanKeyboard_asm ;;escaneamos las teclas
    ld   hl, #Joy0_Fire1        ;; HL = Joy0_Fire1 Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  Joy0_Fire1 == 0
    jr   z, Joy0_Fire12_no_pressed    ;; Saltar si Joy0_Fire1 == 1 (Joy0_Fire1 no esta pulsada)
        ld a, #1 ; si presionado
        ret
    Joy0_Fire12_no_pressed:
        ld a, #0 ; no presionado
        ret

ret

;;seleccion de controles
ComprobarControles::
    call cpct_scanKeyboard_asm   ;;escaneamos las teclas
    ;;TECLA 1
    ld   hl, #Key_1              ;; HL = Key_1 Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, key1_no_pressed     ;; Saltar si A == 1 (Key_1 no esta pulsada)
        ; si presionado
        ld  a, #1
        ld  (controll_selected), a
        ret
    key1_no_pressed:

    ;;TECLA 2
    ld   hl, #Key_2              ;; HL = Key_Space Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, key2_no_pressed     ;; Saltar si A == 1 (Key_2 no esta pulsada)
        ; si presionado
        ld  a, #2
        ld  (controll_selected), a
        ret
    key2_no_pressed:

ret

volver_almenu::
    call cpct_scanKeyboard_asm
    ;;Ir al menu

    ld  a, (controll_selected)
    cp #2
    jr  z, es_un_joistick2
        ;;Es un teclado
        ;;Ir al menu
        ld   hl, #Key_E              ;; HL = Key_E Keycode
        call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
        cp   #0                      ;; Mirar si  A == 0
        jr   z, e_not_pressed        ;; Saltar si A == 0 (E no esta pulsada)

            ;;Reiniciar nivel
            jp  cargar_menu

        e_not_pressed:  ;;e no pulsada
        ret
    es_un_joistick2:
        ld   hl, #Joy0_Fire3              ;; HL = Key_E Keycode
        call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
        cp   #0                      ;; Mirar si  A == 0
        jr   z, Joy0_Fire3_not_pressed        ;; Saltar si A == 0 (E no esta pulsada)

            ;;Reiniciar nivel
            jp  cargar_menu

        Joy0_Fire3_not_pressed:  ;;e no pulsada
        ret
;;===============================================================
;;===============================================================
;; PRIVATE FUNCTION
;;===============================================================
;;===============================================================
;;===============================================================
;; Animacion de game over
;; DESTROY:
;;===============================================================
GameoverAnimation:
    ld   a, (desactivar_teclado)
    cp  #1
    jr   z, no_estas_atontao

        ld  a, ent_anim(ix)
        dec a
        ld  ent_anim(ix), a
        cp #0
        jr  z, reiniciar_animacion_atontao
        cp #3
        jp  m, cambiar_animacion_atontao
            ;;Cambiar Estado atontado 1
            ld   b, #5
            ld   c, #12
            call  changeState
            ret
        cambiar_animacion_atontao:
            ;;Cambiar Estado atontado 2
            ld   b, #6
            ld   c, #13
            call  changeState
            ret
        reiniciar_animacion_atontao:
            ;;Reiniciar contador
            ld  a, #6
            ld  ent_anim(ix), a

    no_estas_atontao:
    ret

;;===============================================================
;; Jump control of entity
;; DESTROY:
;;===============================================================
JumpControl:
    ld  a, ent_jmp(ix)      ;; A = entity_jump estado(indice de la matriz)
    cp  #-1                 ;; A == -1? (-1 is not jump)
    ret z                   ;; Si A == 0, haces ret
    cp  #-2                 ;; A == -2? (-2 is not down)
    ret z                   ;; Si A == 0, haces ret

    ;;Cambiar estado de salto solo una vez, si estas en la pos 0 de la matriz
    ld  a, ent_jmp(ix)      ;; A = entity_jump estado(indice de la matriz)
    cp  #0                  ;; Posicion 1 de jumptable
    jr  nz, no_pos_zero     ;; Si no estas en la posicion 0, no cambiar estado

        ;;Estado de subir del salto
        ld   b, #2
        ld   c, #9
        call  changeState

    no_pos_zero:

    ld  a, ent_jmp(ix)      ;; Dejar A = entity_jump

    ;;Obtener el valor del salto
    ld  hl, #jumptable      ;; HL = primera direccion de la tabla de salto
    ld   c, a               ;; |
    ld   b, #0              ;; \ BC = A (offset)
    add hl, bc              ;; HL += BC

    ;;Chocar con el techo
    ld  a, ent_y(ix)        ;; A = entity_y
    cp  #15                 ;; entity_y == 16?
    jr  z, end_of_up        ;; If 16, acabar subida
    cp  #16                 ;; entity_y == 17?
    jr  z, end_of_up        ;; If 17, acabar subida
    cp  #17                 ;; entity_y == 17?
    jr  z, end_of_up        ;; If 17, acabar subida
    cp  #18                 ;; entity_y == 17?
    jr  z, end_of_up        ;; If 17, acabar subida

    ;;Ceck el final del salto
    ld  a, (hl)             ;; A = Jump Movement
    cp  #0x80               ;; Jump vaoue == 0x80?
    jr  z, end_of_up        ;; If 0x80, acabar subida

    ;;Mover la entidad
    ld  b, a                ;; B = Movimiento de Salto
    ld  a, ent_y(ix)        ;; A = entity_y
    add b                   ;; A + B
    ld ent_y(ix), a         ;; Actualizar entity_y

    ;;Incrementar la entidad_jump (indice)
    inc ent_jmp(ix)         ;; entity_jump++

    ret

    ;;Put -2 in el index mientras estas bajando -2
    end_of_up:
        ld ent_gra(ix), #0          ;; Activar gravedad
        ld  ent_jmp(ix), #-2        ;; entity_jump = -2

        ;;Estado bajando en el salto
        ld   b, #3
        ld   c, #10
        call  changeState

    ret

    ;;Put -1 in el index del jump cuando este acabar
    end_of_jump:
        ;;Si ya habia un -1 de antes Saltar para no entrar mas veces
        ld  a, ent_jmp(ix)
        cp  #-1
        ret z

        ld  ent_jmp(ix), #-1      ;; entity_jump = -1

        ;;Estado en el suelo
        ld   b, #0
        ld   c, #7
        call  changeState

    ret

;;===============================================================
;; Gravedad en el heroe
;; DESTROY: AF
;;===============================================================
gravityHero:

    ld  a, ent_gra(ix)     ;;Gravedad
    cp  #1                 ;;A=1: Gravedad desactivada
    jr z, not_grav         ;;Si A==1, not_gravity

        ld  a, ent_y(ix)      ;;|
        cp  #176              ;;|
        jr z, end_of_jump     ;;\Si ent_y==172 estas ya estas abajo del todo, end_of_jump

            ;;Bajar el heroe
            ld   a, ent_y(ix)       ;;|
            add  ent_vy(ix)         ;;|
            ld   ent_y(ix), a       ;;\ entity_x + entity_vx

            ;;Si estas bajando
            ld  a, ent_jmp(ix)
            cp  #-2
            jr  z, cambia_estadobaja

            ;;Si no estas en el suelo
            ld  a, ent_jmp(ix)      ;; A = entity_jump estado(indice de la matriz)
            cp  #-1                 ;; A == -1? (-1 is not jump)
            jr  nz, not_grav

                ld  ent_jmp(ix), #-2        ;; entity_jump = -2

    not_grav:
    ret
    cambia_estadobaja:
        ;;Cambiar Estado bajando en el salto
        ld   b, #3
        ld   c, #10
        call  changeState

        ret

;;===============================================================
;; Mover a la derecha si no esta en el limite
;; DESTROY: AF
;;===============================================================
moveHeroRight:
    ld  a, ent_x(ix)            ;; A = entity_x
    cp  #80 - 12                ;; Mirar si A - 78 == 0 por lo que A == 78, limite derecho
    jr z, d_not_move_r          ;; si A - 78 == 0, no mover a la derecha

        ;;Moverse a la derecha
        ld   a, ent_x(ix)       ;;|
        add  ent_vx(ix)         ;;|
        ld   ent_x(ix), a       ;;\ entity_x + entity_vx

        ;;Si estas bajando
        ld  a, ent_jmp(ix)
        cp  #-2
        jr  z, cambia_direccion_mira

        ;;Si estas saltando no cambiar el estado aqui
        ld  a, ent_jmp(ix)      ;; A = entity_jump
        cp  #-1                 ;; A == -1? Is jump active?
        jr nz, jump_move_r     ;; Jump active, sprite salto

        cambia_direccion_mira:
        ;;Donde mira la entidad
        ld  ent_dir(ix), #0     ;; Actualizar donde mira (0:mira_derecha)

        ;;Cambiar estado de animacion mas lento, con un contadoranima_hero
        ;;Si el contador no ha terminado no se cambia la animacion
        ld   a, ent_cd(ix)
        dec  a
        ld   ent_cd(ix), a
        cp  #0
        jr  nz, no_cambiar_animacion_der

            ;;Para que se gire rapido al cambiar a izquierda
            ld  ent_ci(ix), #1

            ;;Cambiar estado de la entidad (entity_state)
            ld  a, ent_sta(ix)      ;; Comprobar que estado hay que actualizar
            cp  #7                  ;; Mirar si A == 7 (suelo_der)
            jr nz, en_suelo_der     ;; Si A!=7, en_suelo_der

                ;; Actualizar a se mueve
                ld  ent_sta(ix), #8 ;; entity_state = 8 (se mueve_der)
                jr  continuar_anim_hero

            en_suelo_der:
            ;; Actualizar a suelo
            ld  ent_sta(ix), #7    ;; entity_state =  7 (suelo_der)
            continuar_anim_hero:

            ;; Reiniciar contador
            ld   ent_cd(ix), #5

            ret

        no_cambiar_animacion_der:
        ret
    jump_move_r:
    ld  ent_dir(ix), #0
    ld  ent_sta(ix), #9
    d_not_move_r:
    ret

;;===============================================================
;; Mover a la izquierda si no esta en el limite
;; DESTROY: AF
;;===============================================================
moveHeroLeft:
    ld  a, ent_x(ix)            ;; A = entity_x
    cp  #4                      ;; Mirar si A == 0 , limite izquierdo
    jr z, d_not_move_l          ;; si A == 0, no mover a la izquierda

        ;;Moverse a la izquierda
        ld   a, ent_x(ix)       ;;|
        sub  ent_vx(ix)         ;;|
        ld   ent_x(ix), a       ;;\ entity_x + entity_vx

        ;;Si estas bajando
        ld  a, ent_jmp(ix)
        cp  #-2
        jr  z, cambia_direccion_mira2

        ;;Si estas saltando no cambiar entity_state aqui
        ld  a, ent_jmp(ix)      ;; A = entity_jump
        cp  #-1                 ;; A == -1? Is jump active?
        jr nz, jump_move_l     ;; Jump active, cambiar estado salto

        cambia_direccion_mira2:
        ;;Donde mira la entidad
        ld  ent_dir(ix), #1     ;;Actualizar donde mira (1:mira_izquierda)

        ;;Cambiar estado de animacion mas lento, con un contadoranima_hero
        ;;Si el contador no ha terminado no se cambia la animacion
        ld   a, ent_ci(ix)
        dec  a
        ld   ent_ci(ix), a
        cp  #0
        jr  nz, no_cambiar_animacion_izq

            ;;Para que se gire rapido al cambiar a izquierda
            ld  ent_cd(ix), #1

            ;;Cambiar estado de la entidad (entity_state)
            ld a, ent_sta(ix)       ;; Comprobar que estado hay que actualizar
            cp  #0                  ;; Mirar si A == 0 (suelo_izq)
            jr nz, en_suelo_izq     ;; Si A!=0, en_suelo_izq

                ;; Actualizar a se mueve
                ld  ent_sta(ix), #1 ;; entity_state = 1 (se mueve_izq)
                jr  continuar_anim_hero2

            en_suelo_izq:
            ;; Actualizar a suelo
            ld  ent_sta(ix), #0 ;; entity_state = 0 (suelo_izq)
            continuar_anim_hero2:

            ;; Reiniciar contador
            ld   ent_ci(ix), #5

            ret
        no_cambiar_animacion_izq:
        ret
    jump_move_l:
    ld  ent_dir(ix), #1
    ld  ent_sta(ix), #2
    d_not_move_l:
    ret

;;===============================================================
;; Salto del heroe
;; DESTROY: AF
;;===============================================================
startJump:
    ld  a, (hero_jump)  ;; A = hero_jump
    cp  #-1             ;; A == -1? Is jump active?
    ret nz              ;; Jump active, return

        ;;Jump esta activo
        ld  a, #0
        ld (hero_jump), a

    ret


;;===============================================================
;; Checkea el Input por teclado del usuario
;; DESTROY: A, D, BC, HL
;;===============================================================
checkUserInput:
    ;;Escanea el teclado
    call cpct_scanKeyboard_asm

    ld   a, (desactivar_teclado)
    cp  #0
    jr   z, no_poder_jugar

    ;;Comprobar que tecla P esta pulsada
    ld   hl, #Key_P              ;; HL = Key_P Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si A == 0
    jr   z, p_not_pressed        ;; Saltar si A == 0 (P no esta pulsada)

        call moveHeroRight

    p_not_pressed:

    ;;Comprobar que tecla O esta pulsada
    ld   hl, #Key_O              ;; HL = Key_O Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si A == 0
    jr   z, o_not_pressed        ;; Saltar si A == 0 (O no esta pulsada)

        call moveHeroLeft

    o_not_pressed:  ;;A no pulsada

    ;;Comprobar que tecla Q esta pulsada
    ld   hl, #Key_Q              ;; HL = Key_Q Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, q_not_pressed        ;; Saltar si A == 0 (Q no esta pulsada)

        ;;W esta pulsada
        call startJump

    q_not_pressed:  ;;W no pulsada

    ;;Comprobar que tecla A esta pulsada
    ld   hl, #Key_A              ;; HL = Key_A Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, a_not_pressed        ;; Saltar si A == 0 (A no esta pulsada)

        ;;Si la gravedad esta desactivada
        ld  a, ent_gra(ix)
        cp  #0
        jr  z, a_not_pressed

            ;;S pulsada
            ld  ent_gra(ix), #0     ;; Desactivar gravedad

    a_not_pressed:  ;;S no pulsada

    ;;TECLA ESPACIO
    ;;Si el contador no ha terminado no puedes pulsar espacio
    ld   a, (contadorespacio)
    cp  #0
    jr   z, no_decrementar_mas      ;; mantener contador en 0
        dec  a
    no_decrementar_mas:
    ld   (contadorespacio), a
    cp  #0
    jr  nz, space_not_pressed

    ;;Comprobar que tecla Espacio esta pulsada
    ld   hl, #Key_Space          ;; HL = Key_Space Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  Space == 0
    jr   z, space_not_pressed    ;; Saltar si Space == 0 (Space no esta pulsada)

            ;;Space pulsada
            call heroBubble

            ;; Reiniciar contador
            ld   a, #15
            ld   (contadorespacio), a

    space_not_pressed:  ;;Space no pulsada
    ret
    ;;-------------------------------
    ;;-------------------------------
    ;;-------------------------------
    ;;ESTO SI HAY GAME OVER
    ;;-------------------------------
    ;;-------------------------------
    ;;-------------------------------
    no_poder_jugar:
    ;;Reinicar nivel
    ld   hl, #Key_R              ;; HL = Key_Space Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, r_not_pressed        ;; Saltar si A == 0 (R no esta pulsada)

        ;;Reiniciar nivel
        jp  cargar_mapa

    r_not_pressed:  ;;r no pulsada

    ret

;;===============================================================
;; Checkea el Input por joystick del usuario
;; DESTROY: A, D, BC, HL
;;===============================================================
checkUserInputjoy:
    ;;Escanea el teclado
    call cpct_scanKeyboard_asm

    ld   a, (desactivar_teclado)
    cp  #0
    jr   z, no_poder_jugar2

    ;;Comprobar que tecla Joy0_Right esta pulsada
    ld   hl, #Joy0_Right              ;; HL = Key_P Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si A == 0
    jr   z, Joy0_Right_not_pressed        ;; Saltar si A == 0 (P no esta pulsada)

        call moveHeroRight

    Joy0_Right_not_pressed:

    ;;Comprobar que tecla Joy0_Left esta pulsada
    ld   hl, #Joy0_Left              ;; HL = Key_O Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si A == 0
    jr   z, Joy0_Left_not_pressed        ;; Saltar si A == 0 (O no esta pulsada)

        call moveHeroLeft

    Joy0_Left_not_pressed:  ;;A no pulsada

    ;;Comprobar que tecla Q esta pulsada
    ld   hl, #Joy0_Up              ;; HL = Key_Q Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, Joy0_Up_not_pressed        ;; Saltar si A == 0 (Q no esta pulsada)

        ;;W esta pulsada
        call startJump

    Joy0_Up_not_pressed:  ;;W no pulsada

    ;;Comprobar que tecla A esta pulsada
    ld   hl, #Joy0_Down              ;; HL = Key_A Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, Joy0_Down_not_pressed        ;; Saltar si A == 0 (A no esta pulsada)

        ;;Si la gravedad esta desactivada
        ld  a, ent_gra(ix)
        cp  #0
        jr  z, Joy0_Down_not_pressed

            ;;S pulsada
            ld  ent_gra(ix), #0     ;; Desactivar gravedad

    Joy0_Down_not_pressed:  ;;S no pulsada

    ;;TECLA ESPACIO
    ;;Si el contador no ha terminado no puedes pulsar espacio
    ld   a, (contadorespacio)
    cp  #0
    jr   z, no_decrementar_mas2      ;; mantener contador en 0
        dec  a
    no_decrementar_mas2:
    ld   (contadorespacio), a
    cp  #0
    jr  nz, Joy0_Fire1_not_pressed

    ;;Comprobar que tecla Joy0_Fire1 esta pulsada
    ld   hl, #Joy0_Fire1          ;; HL = Key_Space Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  Space == 0
    jr   z, Joy0_Fire1_not_pressed    ;; Saltar si Space == 0 (Space no esta pulsada)

            ;;Space pulsada
            call heroBubble

            ;; Reiniciar contador
            ld   a, #15
            ld   (contadorespacio), a

    Joy0_Fire1_not_pressed:  ;;Space no pulsada
    ret
    ;;-------------------------------
    ;;-------------------------------
    ;;-------------------------------
    ;;ESTO SI HAY GAME OVER
    ;;-------------------------------
    ;;-------------------------------
    ;;-------------------------------
    no_poder_jugar2:
    ;;Reinicar nivel
    ld   hl, #Joy0_Fire2              ;; HL = Key_Space Keycode
    call cpct_isKeyPressed_asm   ;; Si la Tecla pulsada
    cp   #0                      ;; Mirar si  A == 0
    jr   z, Joy0_Fire2_not_pressed        ;; Saltar si A == 0 (R no esta pulsada)

        ;;Reiniciar nivel
        jp  cargar_mapa

    Joy0_Fire2_not_pressed:  ;;r no pulsada

    ret

;;===============================================================
;; Devolver sprite dependiendo del estado del heroe
;; Return:
;;          HL -> Pointer to Sprite
;; DESTROY: AF, HL
;;===============================================================
selectSprite:
    ;;Comprobar estado de animaciones
    ld  a, ent_sta(ix)          ;; A = entity_state

    cp  #0                      ;; A == 0? (0 esta_suelo_izq)
    jr  z, esta_suelo_izq

    cp  #1                      ;; A == 1? (1 esta_moviendo_izq)
    jr  z, esta_moviendo_izq

    cp  #2                      ;; A == 2? (2 esta_saltando_izq)
    jr  z, esta_saltando_izq

    cp  #3                      ;; A == 3? (3 esta_bajando_izq)
    jr  z, esta_bajando_izq

    cp  #4                      ;; A == 4? (4 esta_disparando_izq)
    jr  z, esta_disparando_izq

    cp  #5                      ;; A == 5? (5 esta_atontado1_izq)
    jr  z, esta_atontado1_izq

    cp  #6                      ;; A == 6? (6 esta_atontado2_izq)
    jr  z, esta_atontado2_izq

    cp  #7                      ;; A == 7? (7 esta_suelo_der )
    jr  z, esta_suelo_der

    cp  #8                      ;; A == 8? (8 esta_moviendo_der)
    jr  z, esta_moviendo_der

    cp  #9                      ;; A == 9? (9 esta_saltando_der)
    jr  z, esta_saltando_der

    cp  #10                     ;; A == 10 (10 esta_bajando_der)
    jr  z, esta_bajando_der

    cp  #11                     ;; A == 11 (11 esta_disparando_der)
    jr  z, esta_disparando_der

    cp  #12                     ;; A == 11 (11 esta_atontado1_der)
    jr  z, esta_atontado1_der

    cp  #13                     ;; A == 11 (11 esta_atontado2_der)
    jr  z, esta_atontado2_der

    ;;Dibujar sprite dependiendo del hero_state
    esta_suelo_izq:
    ld  hl, #_snake_sp_00        ;;HL = Sprite
    ret

    esta_moviendo_izq:
    ld  hl, #_snake_sp_01        ;;HL = Sprite
    ret

    esta_saltando_izq:
    ld  hl, #_snake_sp_02        ;;HL = Sprite
    ret

    esta_bajando_izq:
    ld  hl, #_snake_sp_03        ;;HL = Sprite
    ret

    esta_disparando_izq:
    ld  hl, #_snake_sp_04        ;;HL = Sprite
    ret

    esta_atontado1_izq:
    ld  hl, #_snake_sp_05        ;;HL = Sprite
    ret

    esta_atontado2_izq:
    ld  hl, #_snake_sp_06        ;;HL = Sprite
    ret

    esta_suelo_der:
    ld  hl, #_snake_sp_07        ;;HL = Sprite
    ret

    esta_moviendo_der:
    ld  hl, #_snake_sp_08        ;;HL = Sprite
    ret

    esta_saltando_der:
    ld  hl, #_snake_sp_09        ;;HL = Sprite
    ret

    esta_bajando_der:
    ld  hl, #_snake_sp_10        ;;HL = Sprite
    ret

    esta_disparando_der:
    ld  hl, #_snake_sp_11        ;;HL = Sprite
    ret

    esta_atontado1_der:
    ld  hl, #_snake_sp_12        ;;HL = Sprite
    ret

    esta_atontado2_der:
    ld  hl, #_snake_sp_13        ;;HL = Sprite
    ret
