;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "cpctelera.h.s"    ;;CPCtelera simbolos
.include "entity.h.s"
.include "hero.h.s"

.globl _heart_sp
.globl _title_sp

.macro defineInterfaceEntity name, x, y, w, h, sprite
    name'_data:
        defineDrawEntity name'_dw, x, y, w, h  ;;Parte dibujable
        name'_sprite:   .dw  sprite
.endm

defineInterfaceEntity heart1 4 0 #0x08 #0x10 #_heart_sp
defineInterfaceEntity heart2 12 0 #0x08 #0x10 #_heart_sp
defineInterfaceEntity heart3 20 0 #0x08 #0x10 #_heart_sp
defineInterfaceEntity titulo 15 40 #0x32 #0x34 #_title_sp

;;Constantes de las estructuras de datos
.equ ent_sp_l, 4
.equ ent_sp_h, 5

tam_interface = 6

string_score: .asciz "00000"
string_gameover: .asciz "  GAME OVER  "
string_gameover_button1: .asciz " R: Restart Level"
string_gameover_button12: .asciz "Fire2: Restart Level"
string_menuselect1: .asciz " 1:Keyboard "
string_menuselect2: .asciz " 2:Joystick "
string_menu1: .asciz "Press Space for"
string_menu12: .asciz "Press Fire1 for"
string_menu2: .asciz  "     play      "
you_string: .asciz "  You "
win_string: .asciz " Win  "
string_tomenu1: .asciz "Press E to go"
string_tomenu12: .asciz "Press Fire3 to go"
string_tomenu2: .asciz "   to menu   "

game_over_time:: .db #0     ;;0: No gameover, 1: Gameover
controll_selected:: .db #0  ;;0: no selected, 1: Keyboard, 2:Joystick
;;===============================================================
;;===============================================================
;; PUBLIC FUNCTION
;;===============================================================
;;===============================================================

;;===============================================================
;; Muestra toda la interfaz
;; DESTROY: AF, HL, BC, DE
;;===============================================================
interface_game::
    call muestra_vida
    call muestra_puntos
    call dibujarGameOver
ret

;; Se llama en el menu
interface_menu::
    call mostrar_mensaje
    ld  a, (controll_selected)
    cp #0
    jr  z, no_selected
        call mostrar_mensaje2
        ret
    no_selected:
    call select_controlls
ret

;; Se llama cuando terminas el juego
interface_win::

    ;;Dibujar String
    ld  iy, #you_string
    ld  d, #4           ;;Colour text
    ld  e, #0           ;;Colour background
    ld  h, #17          ;; X
    ld  l, #70          ;; Y
    ld  a, #1           ;;Buffer delantero
    call drawString

    ;;Dibujar String
    ld  iy, #win_string
    ld  d, #11          ;;Colour text
    ld  e, #0           ;;Colour background
    ld  h, #37          ;; X
    ld  l, #70          ;; Y
    ld  a, #1           ;;Buffer delantero
    call drawString

    call muestra_puntos_finales

    ;;Dibujar go to menu
    ld  a, (controll_selected)
    cp #2
    jr  z, es_un_joistick_gomenu
        ld  iy, #string_tomenu1
        jr es_un_teclado_gomenu
    es_un_joistick_gomenu:
        ld  iy, #string_tomenu12
    es_un_teclado_gomenu:
    ld  d, #15          ;;Colour text
    ld  e, #0           ;;Colour background
    ld  h, #13          ;; X
    ld  l, #130          ;; Y
    ld  a, #1           ;;Buffer delantero
    call drawString

    ld  iy, #string_tomenu2
    ld  d, #15          ;;Colour text
    ld  e, #0           ;;Colour background
    ld  h, #13          ;; X
    ld  l, #150          ;; Y
    ld  a, #1           ;;Buffer delantero
    call drawString

    ;;botones para volver al menu
    call volver_almenu
ret

;;===============================================================
;;===============================================================
;; PRRIVATE FUNCTION
;;===============================================================
;;===============================================================

;;===============================================================
;; Muestra la vida en corazones
;; DESTROY: AF, HL, BC, DE
;;===============================================================
muestra_vida:
    call hero_getPtrIX
    ld  a, 16(ix)
    cp #0
    jr z, no_quedan_corazones

    ld  ix, #heart1_data
    bucle_draw_cor:
        push af              ;; Guardar el contador A

        ;;Dibujar corazon
        ld  a, #0
        ld  h, ent_sp_h(ix)
        ld  l, ent_sp_l(ix)
        call drawEntity

        ld   bc, #tam_interface    ;; tam_interface
        add  ix, bc                ;; hearts_data + tam_interface = siguiente corazon
        pop  af                    ;; Cargar el contador en A
        dec  a
        jr  nz, bucle_draw_cor    ;; Dibujar tantos corazones como 16(ix) vida del heroe
no_quedan_corazones:
ret

;;muestra los mensajes del menu
mostrar_mensaje:

    ;;Dibujar titulo
    ld  ix, #titulo_data

    ;;Calcular la posicion en pantalla
    ld de, #0xC000
    ld  c, ent_x(ix)            ;; C=entity_x
    ld  b, ent_y(ix)            ;; B=entity_y
    call cpct_getScreenPtr_asm
    ex de, hl                   ;; Intercambia hl por de (para dibujar el sprite en esa posicion)
    ld  c, ent_w(ix)            ;; C=entity_w
    ld  b, ent_h(ix)            ;; B=entity_h
    ld  h, ent_sp_h(ix)
    ld  l, ent_sp_l(ix)
    call cpct_drawSprite_asm    ;; Draw Sprite with mask
    ld de, #0x8000
    ld  c, ent_x(ix)            ;; C=entity_x
    ld  b, ent_y(ix)            ;; B=entity_y
    call cpct_getScreenPtr_asm
    ex de, hl                   ;; Intercambia hl por de (para dibujar el sprite en esa posicion)
    ld  c, ent_w(ix)            ;; C=entity_w
    ld  b, ent_h(ix)            ;; B=entity_h
    ld  h, ent_sp_h(ix)
    ld  l, ent_sp_l(ix)
    call cpct_drawSprite_asm    ;; Draw Sprite with mask
ret

mostrar_mensaje2:

ld  a, (controll_selected)
cp #2
jr  z, es_un_joistick_menu
    ld  iy, #string_menu1
    jr es_un_teclado_menu
es_un_joistick_menu:
    ld  iy, #string_menu12
es_un_teclado_menu:

    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #10          ;; X
    ld  l,  #125         ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString

    ld  iy, #string_menu2
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #10          ;; X
    ld  l,  #145         ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString
ret

select_controlls:
    ld  iy, #string_menuselect1
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #16          ;; X
    ld  l,  #130         ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString

    ld  iy, #string_menuselect2
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #16          ;; X
    ld  l,  #150         ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString
ret
;;===============================================================
;; Muestra los puntos
;; DESTROY: AF, HL, BC, DE
;;===============================================================
muestra_puntos:

    ;;Pasar de decimal a ASCI
    call hero_getPtrIX
    ld  l, 18(ix)
    ld  h, 19(ix)
    ld  de, #string_score
    call hexa_to_asci
    ld  iy, #string_score

    ;;Dibujar String Puntos
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #55         ;; X
    ld  l,  #4          ;; Y
    ld  a,  #0          ;; Buffer trasero
    call drawString

ret

muestra_puntos_finales:

    ;;Pasar de decimal a ASCI
    call hero_getPtrIX
    ld  l, 18(ix)
    ld  h, 19(ix)
    ld  de, #string_score
    call hexa_to_asci
    ld  iy, #string_score

    ;;Dibujar String Puntos
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #29         ;; X
    ld  l,  #100          ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString

ret

;;===============================================================
;; Muestra GAME OVER
;; DESTROY: AF, HL, BC, DE
;;===============================================================
dibujarGameOver:
    ;;Si no es game over no dibujar
    ld   a, (game_over_time)
    cp #0
    jp  z, no_dibujar_gameover

    ld  iy, #string_gameover
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #14          ;; X
    ld  l,  #100         ;; Y
    ld  a,  #0          ;; Buffer trasero
    call drawString

    ld  iy, #string_gameover
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #14          ;; X
    ld  l,  #100         ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString

    ld  a, (controll_selected)
    cp #2
    jr  z, es_un_joistick_gameover
        ld  iy, #string_gameover_button1
        jr es_un_teclado_gameover
    es_un_joistick_gameover:
        ld  iy, #string_gameover_button12
    es_un_teclado_gameover:
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #6          ;; X
    ld  l,  #120         ;; Y
    ld  a,  #0          ;; Buffer trasero
    call drawString

    ld  iy, #string_gameover_button1
    ld  d,  #15         ;; Colour text
    ld  e,  #0          ;; Colour background
    ld  h,  #6          ;; X
    ld  l,  #120         ;; Y
    ld  a,  #1          ;; Buffer trasero
    call drawString

    no_dibujar_gameover:
ret

;;===============================================================
;; HL se pasa de decimal a ASCI
;; INPUT: HL: numero hexadecimal
;; DESTROY: AF, HL, BC, DE
;;===============================================================
hexa_to_asci:
    ld   bc, #-10000
    call  Num1
    ld   bc, #-1000
    call  Num1
    ld   bc, #-100
    call  Num1
    ld   c, #-10
    call  Num1
    ld   c, #-1
    Num1:
        ld  a,#'0'-1
    Num2:
        inc a
        add hl,bc
        jr  c,Num2
        sbc hl,bc
        ld (de),a
        inc de
    ret
