;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy	
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "cpctelera.h.s"
.include "keyboard/keyboard.h.s"
.include "map.h.s"
.include "hero.h.s"
.include "bubbles.h.s"
.include "entity.h.s"
.include "plataform.h.s"
.include "enemy.h.s"
.include "obstacles.h.s"
.include "main.h.s"
.include "interfaz.h.s"

.globl  _play_music
.globl  _menu_music

;;Color del borde
.macro setBorder color
    ld  hl, #0x'color'10
    call cpct_setPALColour_asm
.endm

tiempo_espera: .db #120
tiempo_level:  .db #150
nivel_actual:: .db #-1

level0_string: .asciz "  Level 0  "
level1_string: .asciz "  Level 1  "
level2_string: .asciz "  Level 2  "
level3_string: .asciz "  Level 3  "
level4_string: .asciz "  Level 4  "
level5_string: .asciz "  Level 5  "
level6_string: .asciz "  Level 6  "
level7_string: .asciz "  Level 7  "
level8_string: .asciz "  Level 8  "

;;===============================================================
;;===============================================================
;; PUBLIC FUNCTION
;;===============================================================
;;===============================================================

game_level::
    ;;Comprueba si los enemigos estan muertos
    call Estan_todos_death
    ld  a, d
    cp  #1
    jr  nz, estan_vivos
        ;;Comprobar si es game OVER
        ld  a, (game_over_control)
        cp  #1
        jr  z, es_gameover
            ;;NO Es un game over
            ;;Estan muertos, esperar un tiempo para cambiar level
            ld  a, (tiempo_level)
            dec a
            ld  (tiempo_level), a
            cp  #0
            jr  nz, no_cambialevel
                ;;inicializar tiempo de nuevo
                ld  a, #150
                ld  (tiempo_level), a
                ;;Cambia level siguiente
                ld  a, (nivel_actual)
                inc a
                ld  (nivel_actual), a
                jp  cargar_mapa        ;;cargar mapa va al main
            no_cambialevel:
        es_gameover:
    estan_vivos:
    ret

;;===============================================================
;; Select a level
;; INPUT: A: Number of level selected
;;===============================================================
select_Level::
    cp #-1
    jr nz, no_level8
        setBorder 14
        ld  a, #-1
        call map_Decompress
        call map_InitDraw
    ret
    no_level8:
    cp #0
    jr nz, no_level0
        ;; Sky Blue border
        setBorder 17

        ;;Cargar mapa 0
        ld  a, #0
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 0 aqui
        ld  a, #0
        call hero_init
        call bubble_init
        ld  a, #0
        call plat_init
        ld  a, #0
        call initEnemigos
        ;;Dibujar String
        ld  iy, #level0_string
        ld  d, #6           ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    no_level0:
    cp #1
    jr nz, no_level1
        ;; Red border
        setBorder 1C

        ;;Cargar mapa 0
        ld  a, #1
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 1 aqui
        ld  a, #1
        call hero_init
        call bubble_init
        ld  a, #1
        call plat_init
        ld  a, #1
        call initEnemigos
        ld  a, #1
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level1_string
        ld  d, #10           ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    no_level1:
    cp #2
    jr nz, no_level2
        ;; Sky Blue border
        setBorder 17

        ;;Cargar mapa 2
        ld  a, #2
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 2 aqui
        ld  a, #2
        call hero_init
        call bubble_init
        ld  a, #2
        call plat_init
        ld  a, #2
        call initEnemigos
        ld  a, #2
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level2_string
        ld  d, #6           ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    no_level2:
    cp #3
    jr nz, no_level3
        ;; Orange border
        setBorder 0E

        ;;Cargar mapa 3
        ld  a, #3
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 3 aqui
        ld  a, #3
        call hero_init
        call bubble_init
        ld  a, #3
        call plat_init
        ld  a, #3
        call initEnemigos
        ld  a, #3
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level3_string
        ld  d, #14           ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    no_level3:
    cp #4
    jr nz, no_level4
        ;; Red border
        setBorder 1C

        ;;Cargar mapa 3
        ld  a, #4
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 4 aqui
        ld  a, #4
        call hero_init
        call bubble_init
        ld  a, #4
        call plat_init
        ld  a, #4
        call initEnemigos
        ld  a, #4
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level4_string
        ld  d, #10          ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    no_level4:
    cp #5
    jr nz, no_level5
        ;; Gray border
        setBorder 00

        ;;Cargar mapa 5
        ld  a, #5
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 5 aqui
        ld  a, #5
        call hero_init
        call bubble_init
        ld  a, #5
        call plat_init
        ld  a, #5
        call initEnemigos
        ld  a, #5
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level5_string
        ld  d, #11          ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    no_level5:
    cp #6
    jr nz, no_level6
        ;; Orange border
        setBorder 0E

        ;;Cargar mapa 5
        ld  a, #6
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 5 aqui
        ld  a, #6
        call hero_init
        call bubble_init
        ld  a, #6
        call plat_init
        ld  a, #6
        call initEnemigos
        ld  a, #6
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level6_string
        ld  d, #14          ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    ret
    no_level6:
    cp #7
    jr nz, no_level7
        ;; Sky Blue border
        setBorder 17

        ;;Cargar mapa 7
        ld  a, #7
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 7 aqui
        ld  a, #7
        call hero_init
        call bubble_init
        ld  a, #7
        call plat_init
        ld  a, #7
        call initEnemigos
        ld  a, #7
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level7_string
        ld  d, #10          ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    ret
    no_level7:
    cp #8
    jr nz, no_levelfinal
        ;; Gray border
        setBorder 00

        ;;Cargar mapa 8
        ld  a, #8
        call map_Decompress
        call map_InitDraw
        ;;Iniciar cosas del nivel 8 aqui
        ld  a, #8
        call hero_init
        call bubble_init
        ld  a, #8
        call plat_init
        ld  a, #8
        call initEnemigos
        ld  a, #8
        call obstacle_init
        ;;Dibujar String
        ld  iy, #level8_string
        ld  d, #11          ;;Colour text
        ld  e, #0           ;;Colour background
        ;;...
        jp cargar_musica
    ret
    no_levelfinal:
        ;;Poner pantalla en negro
        call InitBuffers

        ;; Black border
        setBorder 14

        ;;Cargar musica de partida
        ld  de, #_menu_music
        call cpct_akp_musicInit_asm

        call interface_win

        menu_loop:
        call ComprobarE         ;; Si e esta pulsado devuelve 1
        cp #1
        jr z, esta_pulsado_espacio
            jr menu_loop
        esta_pulsado_espacio:
            jp cargar_menu      ;;Ir al menu
cargar_musica:
    ;;Continuar dibujando string
    ld  h, #17          ;; X
    ld  l, #100         ;; Y
    ld  a, #1           ;;Buffer delantero
    call drawString

    continuar_espera:
    ld  a, (tiempo_espera)
    dec a
    halt
    ld  (tiempo_espera), a
    cp #0
    jr  nz, continuar_espera
    ld  a, #120
    ld  (tiempo_espera), a

    ;;Dibujar encima el tilemap
    call map_InitDraw

    ;;Cargar musica de partida
    ld  de, #_play_music
    call cpct_akp_musicInit_asm

ret

;;Comprobar tecla Enter
ComprobarE:
    call cpct_scanKeyboard_asm      ;;escaneamos las teclas
    ld   hl, #Key_E                 ;; HL = Key_E Keycode
    call cpct_isKeyPressed_asm      ;; Si la Tecla pulsada
    cp   #0                         ;; Mirar si  Space == 0
    jr   z, e_no_pressed            ;; Saltar si e == 1 (e no esta pulsada)
        ld a, #1 ; si presionado
    ret
    e_no_pressed:
        ld a, #0 ; no presionado
ret
