;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "hero.h.s"
.include "plataform.h.s"
.include "cpctelera.h.s"    ;;CPCtelera simboloss
.include "enemy.h.s"
.include "bubbles.h.s"
.include "entity.h.s"
.include "level.h.s"
.include "iaengine.h.s"
.include "interfaz.h.s"
.include "obstacles.h.s"

.globl  _snake_pal
.globl  _menu_music

vecesmusica: .db #5

;;===============================================================
;; Inicialize game
;;===============================================================
inicialize_game:
    call cpct_disableFirmware_asm   ;; Funcion que desabilita el firmware

    ld   c, #0                      ;; |
    call cpct_setVideoMode_asm      ;; \ Cambiar a modo de video 0 (16 colores)

    ld  hl, #_snake_pal             ;; Paleta guardada en sprites/nombre.c
    ld  de, #16                     ;; Numero de colores de la paleta
    call cpct_setPalette_asm        ;; Funcion para cambiar la paleta

    ;;Llamar isr por interrupcion
    ld  hl, #isr
    call cpct_setInterruptHandler_asm
ret

;;===============================================================
;; Llama a la musica siempre en una interrupcion usando cpct_setInterruptHandler_asm
;;===============================================================
isr:
    ;;intercambia los registros primarios por los secundarios y los guarda
    ;;Estos registros se tienen que guardar para no perderse
    ex   af, af'
    exx
    push af
    push bc
    push de
    push hl
    push iy

    ld   a, (vecesmusica)
    dec  a
    ld   (vecesmusica), a
    jr  nz, no_reproducir_musica
        call cpct_akp_musicPlay_asm  ;;Reproduce la musica
        ld  a, #5
        ld  (vecesmusica), a

    no_reproducir_musica:
    pop iy
    pop hl
    pop de
    pop bc
    pop af
    exx
    ex   af, af'
    ret
;;===============================================================
;; Main program entry
;;===============================================================
_main::
    ld   sp,  #0x8000               ;;Cambiar la pila de sitio para que no se sobreescriba por el buffer

    call inicialize_game
    call InitBuffers

    cargar_menu::

        ;;Inicializar musica del menu
        ld  de, #_menu_music
        call cpct_akp_musicInit_asm

        ;;Inicializar teclas
        ld  a, #0
        ld  (controll_selected), a

    reiniciar_interfaz::
        ld  a, #-1
        ld (nivel_actual), a
        ld  a, (nivel_actual)           ;; A = Nivel a cargar
        call select_Level

        call interface_menu

    jp menu_loop ;;aqui comprobamos que se presione espacio o q para manual
    continuar_cargandomapa::
    ;;Iniciar vida y puntos
    call first_init_hero

    cargar_mapa::
    ;;Cargar mapa
    ld  a, (nivel_actual)           ;; A = Nivel a cargar
    call select_Level

    main_loop:
        ;comprobamos si queda algun enemigo vivo

        call game_level             ;; Cambia de nivel

        call obstacle_update        ;; comprobar colision con obstaculos
        call hero_update            ;; Update our hero
        call bubble_update          ;; Actualizar burbujas
        call enemy_update           ;; Actualiza el enemigo y su ia
        call HayColision            ;; comprueba si colisiona alguna burbuja con algun enemigo

        ;call plataform_draw        ;; Dibujar plataformas
        call enemy_drawGroup
        call bubble_draw            ;; Dibujado de burbujas
        call hero_draw
        call obstacle_draw
        call interface_game

        call cpct_waitVSYNC_asm     ;; Espera a que el raster este fuera de la pantalla (sincronizacion vertical)
        call newScene               ;; Copia el contenido del buffer

    jr main_loop

    menu_loop:
        call cpct_waitVSYNC_asm     ;; Espera a que el raster este fuera de la pantalla (sincronizacion vertical)

        ;;Primero seleccionar controles
        ld  a, (controll_selected)
        cp #0
        jr  z, no_selected_controllers
            ;;Espacio
            call ComprobarEspacio       ;; Si espacio esta pulsado devuelve 1
            cp #1
            jr z, esta_pulsado_espacio
                jr menu_loop
            esta_pulsado_espacio: ;; cambiamos el nivel a 1, luego le pasamos a cargar nivel
                ld a, #0
                ld (nivel_actual), a
                jp continuar_cargandomapa

        no_selected_controllers:
        call ComprobarControles
        ld  a, (controll_selected)
        cp #0
        jr  z, no_selected_controllers2
            jp reiniciar_interfaz
        no_selected_controllers2:
        jr menu_loop

    ;; fin de menu_loop
