;; INFO LICENSE
;;    This file is part of SnakeBubble.
;;    Copyright (C) 2018-2019 David Ramon Sevilla / Jose Vicente Anton Coy	
;;    SnakeBubble is free software: you can redistribute it and/or modify
;;    it under the terms of the GNU General Public License as published by
;;    the Free Software Foundation, either version 3 of the License, or
;;    (at your option) any later version.
;;
;;    SnakeBubble is distributed in the hope that it will be useful,
;;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;    GNU General Public License for more details.
;;
;;    You should have received a copy of the GNU General Public License
;;    along with SnakeBubble.  If not, see <https://www.gnu.org/licenses/>.
;;---------------------------------Fin license info----------------------------------------

.area _DATA
.area _CODE

.include "cpctelera.h.s"
.include "interfaz.h.s"
.include "level.h.s"

.globl _menu_pack_end
.globl _level0_pack_end
.globl _level1_pack_end
.globl _level2_pack_end
.globl _level3_pack_end
.globl _level4_pack_end
.globl _level5_pack_end
.globl _level6_pack_end
.globl _level7_pack_end
.globl _level8_pack_end

_level0_W = 20

decompress_buffer     = 0x040
levelmaxsize          = 0x2F4
decompress_buffer_end = decompress_buffer + levelmaxsize - 1
tileset_ptr           = decompress_buffer + 500
tilemap_ptr           = decompress_buffer + 0

temporal_hl: .dw  #0x00, #0x00

;;===============================================================
;;===============================================================
;; PUBLIC FUNCTION
;;===============================================================
;;===============================================================
;;===============================================================
;; Decompress the tilemap
;; DESTROY: HL, DE
;; INPUT:
;;          A: level selected
;; RETURNS: -
;;===============================================================
map_Decompress::
    cp  #-1
    jr  nz, no_menu
        ld  hl, #_menu_pack_end
        jr  continuar_descomp
    no_menu:
    cp  #0
    jr  nz, no_level0
        ld  hl, #_level0_pack_end
        jr  continuar_descomp
    no_level0:
    cp  #1
    jr  nz, no_level1
        ld  hl, #_level1_pack_end
        jr  continuar_descomp
    no_level1:
    cp  #2
    jr  nz, no_level2
        ld  hl, #_level2_pack_end
        jr  continuar_descomp
    no_level2:
    cp  #3
    jr  nz, no_level3
        ld  hl, #_level3_pack_end
        jr  continuar_descomp
    no_level3:
    cp  #4
    jr  nz, no_level4
        ld  hl, #_level4_pack_end
        jr  continuar_descomp
    no_level4:
    cp  #5
    jr  nz, no_level5
        ld  hl, #_level5_pack_end
        jr  continuar_descomp
    no_level5:
    cp  #6
    jr  nz, no_level6
        ld  hl, #_level6_pack_end
        jr  continuar_descomp
    no_level6:
    cp  #7
    jr  nz, no_level7
        ld  hl, #_level7_pack_end
        jr  continuar_descomp
    no_level7:
    cp  #8
    jr  nz, no_level8
        ld  hl, #_level8_pack_end
        jr  continuar_descomp
    no_level8:
    continuar_descomp:
    ld  de, #decompress_buffer_end
    call cpct_zx7b_decrunch_s_asm

    ret

;;===============================================================
;; Initiation tilemap draw
;; DESTROY: HL, DE
;; RETURNS: -
;;===============================================================
map_InitDraw::
    ;;Inicializar game over
    ld  a, #0
    ld  (game_over_time), a

    ;;Preparar el tilemap
    ld  bc, #0x1914         ;; b=alto c=ancho (20x25 en tiles)
    ld  de, #_level0_W      ;; Tilemap width
    ld  hl, #tileset_ptr    ;; Tileset pointer
    call cpct_etm_setDrawTilemap4x8_ag_asm

    ;;Dibujar el Tilemap
    ld  hl, #0xC000
    call map_Draw

    ;;Dibujar el Tilemap
    ld  hl, #0x8000
    call map_Draw

    ret

;;===============================================================
;; Tilemap draw
;; DESTROY: HL, DE
;; INPUT: HL = Video Memory Pointer
;;===============================================================
map_Draw::
    ld  de, #tilemap_ptr
    call cpct_etm_drawTilemap4x8_ag_asm

    ret

;;===============================================================
;; Set and Draw tilemap
;; DESTROY: HL, DE
;; INPUT: BC, DE
;;===============================================================

SetDrawTileset:
    ld  de, #_level0_W      ;; Tilemap width
    ld  hl, #tileset_ptr    ;; Tileset pointer
    call cpct_etm_setDrawTilemap4x8_ag_asm
    ret

DrawTileset:
    ld  hl, (temporal_hl)
    add hl, bc                  ;;Suma HL + BC
    call cpct_etm_drawTilemap4x8_ag_asm
    ret

;;===============================================================
;; Tilemap object update, depend of level
;; DESTROY: HL, DE
;; INPUT: HL = Video Memory Pointer
;;===============================================================
map_DrawObjects::
    ;;Si es game over no redibujar
    ld   a, (game_over_time)
    cp #1
    jp  z, no_redibujar_tile

    ld (temporal_hl), hl

    ;;suelo
    ld  bc, #0x0114             ;; b=alto c=ancho
    call SetDrawTileset
    ld  bc, #0x0780             ;; Suma a la memoria (20 x 80 por fila y pasar a hexa)
    ld  de, #tilemap_ptr + 480  ;; Suma al puntero cada tile (20 x numero de filas)
    call DrawTileset

    ;;pared derecha
    ld  bc, #0x1601         ;; b=alto c=ancho
    call SetDrawTileset
    ld  bc, #0x00EC
    ld  de, #tilemap_ptr + 59
    call DrawTileset

    ;;pared izquierda
    ld  bc, #0x1601         ;; b=alto c=ancho
    call SetDrawTileset
    ld  bc, #0x00A0
    ld  de, #tilemap_ptr + 40
    call DrawTileset

    ;;techo
    ld  bc, #0x0214         ;; b=alto c=ancho
    call SetDrawTileset
    ld  bc, #0x0000
    ld  de, #tilemap_ptr
    call DrawTileset

    ld  a, (nivel_actual)
    cp #0
    jr nz, no_level0_red

        ;;plataforma1
        ld  bc, #0x0206         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x047C
        ld  de, #tilemap_ptr + 287
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0207         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x05F4
        ld  de, #tilemap_ptr + 381
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0207         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0620
        ld  de, #tilemap_ptr + 392
        call DrawTileset

        jp acabar_red
    no_level0_red:

    ld  a, (nivel_actual)
    cp #1
    jp nz, no_level1_red

        ;;plataforma1
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x05F4
        ld  de, #tilemap_ptr + 381
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0624
        ld  de, #tilemap_ptr + 393
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0464
        ld  de, #tilemap_ptr + 281
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0494
        ld  de, #tilemap_ptr + 293
        call DrawTileset

        ;;plataforma5
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x02D4
        ld  de, #tilemap_ptr + 181
        call DrawTileset

        ;;plataforma6
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0304
        ld  de, #tilemap_ptr + 193
        call DrawTileset

        ;;pinchos
        ld  bc, #0x0114             ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0730             ;; Suma a la memoria (20 x 80 por fila y pasar a hexa)
        ld  de, #tilemap_ptr + 460  ;; Suma al puntero cada tile (20 x numero de filas)
        call DrawTileset

        jp acabar_red
    no_level1_red:

    ld  a, (nivel_actual)
    cp #2
    jp nz, no_level2_red

        ;;plataforma1
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x05F4
        ld  de, #tilemap_ptr + 381
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0520
        ld  de, #tilemap_ptr + 328
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x044C
        ld  de, #tilemap_ptr + 275
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x0114         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0284
        ld  de, #tilemap_ptr + 161
        call DrawTileset

        jp acabar_red
    no_level2_red:

    ld  a, (nivel_actual)
    cp #3
    jp nz, no_level3_red

        ;;plataforma1
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0694
        ld  de, #tilemap_ptr + 421
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0584
        ld  de, #tilemap_ptr + 353
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0108         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0414
        ld  de, #tilemap_ptr + 261
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0284
        ld  de, #tilemap_ptr + 161
        call DrawTileset

        ;;plataforma5
        ld  bc, #0x0108         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x025C
        ld  de, #tilemap_ptr + 151
        call DrawTileset

        jp acabar_red
    no_level3_red:

    ld  a, (nivel_actual)
    cp #4
    jp nz, no_level4_red

        ;;plataforma1
        ld  bc, #0x0102         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0600
        ld  de, #tilemap_ptr + 384
        call DrawTileset

        ;;plataforma2 y pinchos
        ld  bc, #0x020A         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0568
        ld  de, #tilemap_ptr + 346
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x049C
        ld  de, #tilemap_ptr + 295
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0414
        ld  de, #tilemap_ptr + 261
        call DrawTileset

        ;;plataforma5
        ld  bc, #0x020A         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0294
        ld  de, #tilemap_ptr + 165
        call DrawTileset

        jp acabar_red
    no_level4_red:

    ld  a, (nivel_actual)
    cp #5
    jp nz, no_level5_red

        ;;plataforma1
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0600
        ld  de, #tilemap_ptr + 384
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0620
        ld  de, #tilemap_ptr + 392
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0470
        ld  de, #tilemap_ptr + 284
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0490
        ld  de, #tilemap_ptr + 292
        call DrawTileset

        ;;plataforma5
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x02E0
        ld  de, #tilemap_ptr + 184
        call DrawTileset

        ;;plataforma6
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0300
        ld  de, #tilemap_ptr + 192
        call DrawTileset

        ;;techo2
        ld  bc, #0x0114         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x00A8
        ld  de, #tilemap_ptr + 42
        call DrawTileset

        jp acabar_red
    no_level5_red:

    ld  a, (nivel_actual)
    cp #6
    jp nz, no_level6_red

        ;;pinchos
        ld  bc, #0x0114             ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0730             ;; Suma a la memoria (20 x 80 por fila y pasar a hexa)
        ld  de, #tilemap_ptr + 460  ;; Suma al puntero cada tile (20 x numero de filas)
        call DrawTileset

        ;;plataforma1
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x071C
        ld  de, #tilemap_ptr + 455
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x05B0
        ld  de, #tilemap_ptr + 364
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0102         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0414
        ld  de, #tilemap_ptr + 261
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x010E         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x03D0
        ld  de, #tilemap_ptr + 244
        call DrawTileset

        ;;plataforma5
        ld  bc, #0x0102         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0274
        ld  de, #tilemap_ptr + 157
        call DrawTileset

        ;;plataforma6
        ld  bc, #0x010E         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x01E8
        ld  de, #tilemap_ptr + 122
        call DrawTileset

        jp acabar_red

    no_level6_red:
    ld  a, (nivel_actual)
    cp #7
    jp nz, no_level7_red

        ;;pinchos
        ld  bc, #0x0114             ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0730             ;; Suma a la memoria (20 x 80 por fila y pasar a hexa)
        ld  de, #tilemap_ptr + 460  ;; Suma al puntero cada tile (20 x numero de filas)
        call DrawTileset

        ;;plataforma1
        ld  bc, #0x0105         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0284
        ld  de, #tilemap_ptr + 161
        call DrawTileset

        ;;plataforma2
        ld  bc, #0x0105         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x02B8
        ld  de, #tilemap_ptr + 174
        call DrawTileset

        ;;plataforma3
        ld  bc, #0x0106         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x03DC
        ld  de, #tilemap_ptr + 247
        call DrawTileset

        ;;plataforma4
        ld  bc, #0x0105         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0504
        ld  de, #tilemap_ptr + 321
        call DrawTileset

        ;;plataforma5
        ld  bc, #0x0105         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0538
        ld  de, #tilemap_ptr + 334
        call DrawTileset

        ;;plataforma6
        ld  bc, #0x0104         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0660
        ld  de, #tilemap_ptr + 408
        call DrawTileset

        jp acabar_red

    no_level7_red:
    ld  a, (nivel_actual)
    cp #8
    jp nz, no_level8_red

        ;;pinchos
        ld  bc, #0x0114             ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0730             ;; Suma a la memoria (20 x 80 por fila y pasar a hexa)
        ld  de, #tilemap_ptr + 460  ;; Suma al puntero cada tile (20 x numero de filas)
        call DrawTileset

        ;;plataforma1
        ld  bc, #0x0102         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0704
        ld  de, #tilemap_ptr + 449
        call DrawTileset

        ;;plataforma2 con pinchos
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x05A4
        ld  de, #tilemap_ptr + 361
        call DrawTileset

        ;;plataforma3 con pinchos
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x05DC
        ld  de, #tilemap_ptr + 375
        call DrawTileset

        ;;plataforma4 con pinchos
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x04D0
        ld  de, #tilemap_ptr + 308
        call DrawTileset

        ;;plataforma5 con pinchos
        ld  bc, #0x0204         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x0340
        ld  de, #tilemap_ptr + 208
        call DrawTileset

        ;;plataforma6
        ld  bc, #0x0114         ;; b=alto c=ancho
        call SetDrawTileset
        ld  bc, #0x01E4
        ld  de, #tilemap_ptr + 121
        call DrawTileset

        jp acabar_red

    no_level8_red:

    no_redibujar_tile:
    acabar_red:
    ret
